@@ -41,6 +41,10 @@ G_BEGIN_DECLS
  * IPC payloads to be sent in `FlightData` protobuf messages by
  * #GArrowRecordBatchReader`.
  *
+ * #GAFlightMessageReader is a class for IPC payloads uploaded by a
+ * client. Also allows reading application-defined metadata via the
+ * Flight protocol.
+ *
  * #GAFlightServerAuthSender is a class for sending messages to the
  * client during an authentication handshake.
  *
@@ -59,8 +63,8 @@ G_BEGIN_DECLS
  * Since: 5.0.0
  */
 
-
-typedef struct GAFlightDataStreamPrivate_ {
+typedef struct GAFlightDataStreamPrivate_
+{
   arrow::flight::FlightDataStream *stream;
 } GAFlightDataStreamPrivate;
 
@@ -68,14 +72,11 @@ enum {
   PROP_STREAM = 1,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightDataStream,
-                           gaflight_data_stream,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightDataStream, gaflight_data_stream, G_TYPE_OBJECT)
 
-#define GAFLIGHT_DATA_STREAM_GET_PRIVATE(obj)        \
-  static_cast<GAFlightDataStreamPrivate *>(          \
-    gaflight_data_stream_get_instance_private(       \
-      GAFLIGHT_DATA_STREAM(obj)))
+#define GAFLIGHT_DATA_STREAM_GET_PRIVATE(obj)                                            \
+  static_cast<GAFlightDataStreamPrivate *>(                                              \
+    gaflight_data_stream_get_instance_private(GAFLIGHT_DATA_STREAM(obj)))
 
 static void
 gaflight_data_stream_finalize(GObject *object)
@@ -97,8 +98,8 @@ gaflight_data_stream_set_property(GObject *object,
 
   switch (prop_id) {
   case PROP_STREAM:
-    priv->stream = static_cast<arrow::flight::FlightDataStream *>(
-      g_value_get_pointer(value));
+    priv->stream =
+      static_cast<arrow::flight::FlightDataStream *>(g_value_get_pointer(value));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -120,16 +121,16 @@ gaflight_data_stream_class_init(GAFlightDataStreamClass *klass)
   gobject_class->set_property = gaflight_data_stream_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("stream",
-                              "Stream",
-                              "The raw arrow::flight::FlightDataStream *",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "stream",
+    "Stream",
+    "The raw arrow::flight::FlightDataStream *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_STREAM, spec);
 }
 
-
-typedef struct GAFlightRecordBatchStreamPrivate_ {
+typedef struct GAFlightRecordBatchStreamPrivate_
+{
   GArrowRecordBatchReader *reader;
 } GAFlightRecordBatchStreamPrivate;
 
@@ -141,9 +142,9 @@ G_DEFINE_TYPE_WITH_PRIVATE(GAFlightRecordBatchStream,
                            gaflight_record_batch_stream,
                            GAFLIGHT_TYPE_DATA_STREAM)
 
-#define GAFLIGHT_RECORD_BATCH_STREAM_GET_PRIVATE(obj)        \
-  static_cast<GAFlightRecordBatchStreamPrivate *>(           \
-    gaflight_record_batch_stream_get_instance_private(       \
+#define GAFLIGHT_RECORD_BATCH_STREAM_GET_PRIVATE(obj)                                    \
+  static_cast<GAFlightRecordBatchStreamPrivate *>(                                       \
+    gaflight_record_batch_stream_get_instance_private(                                   \
       GAFLIGHT_RECORD_BATCH_STREAM(obj)))
 
 static void
@@ -217,12 +218,12 @@ gaflight_record_batch_stream_class_init(GAFlightRecordBatchStreamClass *klass)
    *
    * Since: 6.0.0
    */
-  spec = g_param_spec_object("reader",
-                             "Reader",
-                             "The reader that produces record batches",
-                             GARROW_TYPE_RECORD_BATCH_READER,
-                             static_cast<GParamFlags>(G_PARAM_READWRITE |
-                                                      G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_object(
+    "reader",
+    "Reader",
+    "The reader that produces record batches",
+    GARROW_TYPE_RECORD_BATCH_READER,
+    static_cast<GParamFlags>(G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_READER, spec);
 }
 
@@ -248,19 +249,53 @@ gaflight_record_batch_stream_new(GArrowRecordBatchReader *reader,
   } else {
     arrow_options = &arrow_options_default;
   }
-  auto stream = std::make_unique<
-    arrow::flight::RecordBatchStream>(arrow_reader, *arrow_options);
+  auto stream =
+    std::make_unique<arrow::flight::RecordBatchStream>(arrow_reader, *arrow_options);
   return static_cast<GAFlightRecordBatchStream *>(
     g_object_new(GAFLIGHT_TYPE_RECORD_BATCH_STREAM,
-                 "stream", stream.release(),
-                 "reader", reader,
+                 "stream",
+                 stream.release(),
+                 "reader",
+                 reader,
                  NULL));
 }
 
+G_DEFINE_TYPE(GAFlightMessageReader,
+              gaflight_message_reader,
+              GAFLIGHT_TYPE_RECORD_BATCH_READER)
 
-typedef struct GAFlightServerCallContextPrivate_ {
+static void
+gaflight_message_reader_init(GAFlightMessageReader *object)
+{
+}
+
+static void
+gaflight_message_reader_class_init(GAFlightMessageReaderClass *klass)
+{
+}
+
+/**
+ * gaflight_message_reader_get_descriptor:
+ * @reader: A #GAFlightMessageReader.
+ *
+ * Returns: (transfer full): The descriptor for this upload.
+ *
+ * Since: 14.0.0
+ */
+GAFlightDescriptor *
+gaflight_message_reader_get_descriptor(GAFlightMessageReader *reader)
+{
+  auto flight_reader = gaflight_message_reader_get_raw(reader);
+  const auto &flight_descriptor = flight_reader->descriptor();
+  return gaflight_descriptor_new_raw(&flight_descriptor);
+}
+
+struct GAFlightServerCallContextPrivate
+{
   arrow::flight::ServerCallContext *call_context;
-} GAFlightServerCallContextPrivate;
+  std::string current_incoming_header_key;
+  std::string current_incoming_header_value;
+};
 
 enum {
   PROP_CALL_CONTEXT = 1,
@@ -270,11 +305,20 @@ G_DEFINE_TYPE_WITH_PRIVATE(GAFlightServerCallContext,
                            gaflight_server_call_context,
                            G_TYPE_OBJECT)
 
-#define GAFLIGHT_SERVER_CALL_CONTEXT_GET_PRIVATE(obj)   \
-  static_cast<GAFlightServerCallContextPrivate *>(      \
-    gaflight_server_call_context_get_instance_private(  \
+#define GAFLIGHT_SERVER_CALL_CONTEXT_GET_PRIVATE(obj)                                    \
+  static_cast<GAFlightServerCallContextPrivate *>(                                       \
+    gaflight_server_call_context_get_instance_private(                                   \
       GAFLIGHT_SERVER_CALL_CONTEXT(obj)))
 
+static void
+gaflight_server_call_context_finalize(GObject *object)
+{
+  auto priv = GAFLIGHT_SERVER_CALL_CONTEXT_GET_PRIVATE(object);
+  priv->current_incoming_header_key.~basic_string();
+  priv->current_incoming_header_value.~basic_string();
+  G_OBJECT_CLASS(gaflight_server_call_context_parent_class)->finalize(object);
+}
+
 static void
 gaflight_server_call_context_set_property(GObject *object,
                                           guint prop_id,
@@ -286,8 +330,7 @@ gaflight_server_call_context_set_property(GObject *object,
   switch (prop_id) {
   case PROP_CALL_CONTEXT:
     priv->call_context =
-      static_cast<arrow::flight::ServerCallContext *>(
-        g_value_get_pointer(value));
+      static_cast<arrow::flight::ServerCallContext *>(g_value_get_pointer(value));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -298,6 +341,9 @@ gaflight_server_call_context_set_property(GObject *object,
 static void
 gaflight_server_call_context_init(GAFlightServerCallContext *object)
 {
+  auto priv = GAFLIGHT_SERVER_CALL_CONTEXT_GET_PRIVATE(object);
+  new (&(priv->current_incoming_header_key)) std::string;
+  new (&(priv->current_incoming_header_value)) std::string;
 }
 
 static void
@@ -305,19 +351,46 @@ gaflight_server_call_context_class_init(GAFlightServerCallContextClass *klass)
 {
   auto gobject_class = G_OBJECT_CLASS(klass);
 
+  gobject_class->finalize = gaflight_server_call_context_finalize;
   gobject_class->set_property = gaflight_server_call_context_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("call-context",
-                              "Call context",
-                              "The raw arrow::flight::ServerCallContext",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "call-context",
+    "Call context",
+    "The raw arrow::flight::ServerCallContext",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_CALL_CONTEXT, spec);
 }
 
+/**
+ * gaflight_server_call_context_foreach_incoming_header:
+ * @context: A #GAFlightServerCallContext.
+ * @func: (scope call): The user's callback function.
+ * @user_data: (closure): Data for @func.
+ *
+ * Iterates over all incoming headers.
+ *
+ * Since: 14.0.0
+ */
+void
+gaflight_server_call_context_foreach_incoming_header(GAFlightServerCallContext *context,
+                                                     GAFlightHeaderFunc func,
+                                                     gpointer user_data)
+{
+  auto priv = GAFLIGHT_SERVER_CALL_CONTEXT_GET_PRIVATE(context);
+  auto flight_context = gaflight_server_call_context_get_raw(context);
+  for (const auto &header : flight_context->incoming_headers()) {
+    priv->current_incoming_header_key = std::string(header.first);
+    priv->current_incoming_header_value = std::string(header.second);
+    func(priv->current_incoming_header_key.c_str(),
+         priv->current_incoming_header_value.c_str(),
+         user_data);
+  }
+}
 
-struct GAFlightServerAuthSenderPrivate {
+struct GAFlightServerAuthSenderPrivate
+{
   arrow::flight::ServerAuthSender *sender;
 };
 
@@ -329,10 +402,9 @@ G_DEFINE_TYPE_WITH_PRIVATE(GAFlightServerAuthSender,
                            gaflight_server_auth_sender,
                            G_TYPE_OBJECT)
 
-#define GAFLIGHT_SERVER_AUTH_SENDER_GET_PRIVATE(obj)        \
-  static_cast<GAFlightServerAuthSenderPrivate *>(           \
-    gaflight_server_auth_sender_get_instance_private(       \
-      GAFLIGHT_SERVER_AUTH_SENDER(obj)))
+#define GAFLIGHT_SERVER_AUTH_SENDER_GET_PRIVATE(obj)                                     \
+  static_cast<GAFlightServerAuthSenderPrivate *>(                                        \
+    gaflight_server_auth_sender_get_instance_private(GAFLIGHT_SERVER_AUTH_SENDER(obj)))
 
 static void
 gaflight_server_auth_sender_set_property(GObject *object,
@@ -366,11 +438,11 @@ gaflight_server_auth_sender_class_init(GAFlightServerAuthSenderClass *klass)
   gobject_class->set_property = gaflight_server_auth_sender_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("sender",
-                              "Sender",
-                              "The raw arrow::flight::ServerAuthSender *",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "sender",
+    "Sender",
+    "The raw arrow::flight::ServerAuthSender *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_SERVER_AUTH_SENDER, spec);
 }
 
@@ -399,8 +471,8 @@ gaflight_server_auth_sender_write(GAFlightServerAuthSender *sender,
   return garrow::check(error, status, "[flight-server-auth-sender][write]");
 }
 
-
-struct GAFlightServerAuthReaderPrivate {
+struct GAFlightServerAuthReaderPrivate
+{
   arrow::flight::ServerAuthReader *reader;
 };
 
@@ -412,10 +484,9 @@ G_DEFINE_TYPE_WITH_PRIVATE(GAFlightServerAuthReader,
                            gaflight_server_auth_reader,
                            G_TYPE_OBJECT)
 
-#define GAFLIGHT_SERVER_AUTH_READER_GET_PRIVATE(obj)        \
-  static_cast<GAFlightServerAuthReaderPrivate *>(           \
-    gaflight_server_auth_reader_get_instance_private(       \
-      GAFLIGHT_SERVER_AUTH_READER(obj)))
+#define GAFLIGHT_SERVER_AUTH_READER_GET_PRIVATE(obj)                                     \
+  static_cast<GAFlightServerAuthReaderPrivate *>(                                        \
+    gaflight_server_auth_reader_get_instance_private(GAFLIGHT_SERVER_AUTH_READER(obj)))
 
 static void
 gaflight_server_auth_reader_set_property(GObject *object,
@@ -449,11 +520,11 @@ gaflight_server_auth_reader_class_init(GAFlightServerAuthReaderClass *klass)
   gobject_class->set_property = gaflight_server_auth_reader_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("reader",
-                              "Reader",
-                              "The raw arrow::flight::ServerAuthReader *",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "reader",
+    "Reader",
+    "The raw arrow::flight::ServerAuthReader *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_SERVER_AUTH_READER, spec);
 }
 
@@ -470,8 +541,7 @@ gaflight_server_auth_reader_class_init(GAFlightServerAuthReaderClass *klass)
  * Since: 12.0.0
  */
 GBytes *
-gaflight_server_auth_reader_read(GAFlightServerAuthReader *reader,
-                                 GError **error)
+gaflight_server_auth_reader_read(GAFlightServerAuthReader *reader, GError **error)
 {
   auto flight_reader = gaflight_server_auth_reader_get_raw(reader);
   std::string flight_message;
@@ -482,8 +552,8 @@ gaflight_server_auth_reader_read(GAFlightServerAuthReader *reader,
   return g_bytes_new(flight_message.data(), flight_message.size());
 }
 
-
-struct GAFlightServerAuthHandlerPrivate {
+struct GAFlightServerAuthHandlerPrivate
+{
   std::shared_ptr<arrow::flight::ServerAuthHandler> handler;
 };
 
@@ -495,9 +565,9 @@ G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE(GAFlightServerAuthHandler,
                                     gaflight_server_auth_handler,
                                     G_TYPE_OBJECT)
 
-#define GAFLIGHT_SERVER_AUTH_HANDLER_GET_PRIVATE(obj)        \
-  static_cast<GAFlightServerAuthHandlerPrivate *>(           \
-    gaflight_server_auth_handler_get_instance_private(       \
+#define GAFLIGHT_SERVER_AUTH_HANDLER_GET_PRIVATE(obj)                                    \
+  static_cast<GAFlightServerAuthHandlerPrivate *>(                                       \
+    gaflight_server_auth_handler_get_instance_private(                                   \
       GAFLIGHT_SERVER_AUTH_HANDLER(obj)))
 
 static void
@@ -519,9 +589,8 @@ gaflight_server_auth_handler_set_property(GObject *object,
   switch (prop_id) {
   case PROP_SERVER_AUTH_HANDLER:
     if (g_value_get_pointer(value)) {
-      priv->handler =
-        *static_cast<std::shared_ptr<arrow::flight::ServerAuthHandler>*>(
-          g_value_get_pointer(value));
+      priv->handler = *static_cast<std::shared_ptr<arrow::flight::ServerAuthHandler> *>(
+        g_value_get_pointer(value));
     }
     break;
   default:
@@ -534,7 +603,7 @@ static void
 gaflight_server_auth_handler_init(GAFlightServerAuthHandler *object)
 {
   auto priv = GAFLIGHT_SERVER_AUTH_HANDLER_GET_PRIVATE(object);
-  new(&priv->handler) std::shared_ptr<arrow::flight::ServerAuthHandler>;
+  new (&priv->handler) std::shared_ptr<arrow::flight::ServerAuthHandler>;
 }
 
 static void
@@ -546,29 +615,30 @@ gaflight_server_auth_handler_class_init(GAFlightServerAuthHandlerClass *klass)
   gobject_class->set_property = gaflight_server_auth_handler_set_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("handler",
-                              "Handler",
-                              "The raw std::shared_ptr<"
-                              "arrow::flight::ServerAuthHandler>",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "handler",
+    "Handler",
+    "The raw std::shared_ptr<"
+    "arrow::flight::ServerAuthHandler>",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_SERVER_AUTH_HANDLER, spec);
 }
 
-
 G_END_DECLS
 namespace gaflight {
   class ServerCustomAuthHandler : public arrow::flight::ServerAuthHandler {
   public:
     explicit ServerCustomAuthHandler(GAFlightServerCustomAuthHandler *handler)
       : arrow::flight::ServerAuthHandler(),
-        handler_(handler) {
+        handler_(handler)
+    {
     }
 
     arrow::Status
     Authenticate(const arrow::flight::ServerCallContext &context,
                  arrow::flight::ServerAuthSender *sender,
-                 arrow::flight::ServerAuthReader *reader) override {
+                 arrow::flight::ServerAuthReader *reader) override
+    {
       auto klass = GAFLIGHT_SERVER_CUSTOM_AUTH_HANDLER_GET_CLASS(handler_);
       auto gacontext = gaflight_server_call_context_new_raw(&context);
       auto gasender = gaflight_server_auth_sender_new_raw(sender);
@@ -591,22 +661,20 @@ namespace gaflight {
     arrow::Status
     IsValid(const arrow::flight::ServerCallContext &context,
             const std::string &token,
-            std::string *peer_identity) override {
+            std::string *peer_identity) override
+    {
       auto klass = GAFLIGHT_SERVER_CUSTOM_AUTH_HANDLER_GET_CLASS(handler_);
       auto gacontext = gaflight_server_call_context_new_raw(&context);
       auto gtoken = g_bytes_new_static(token.data(), token.size());
-      GBytes *gpeer_identity = nullptr;
       GError *error = nullptr;
-      klass->is_valid(handler_, gacontext, gtoken, &gpeer_identity, &error);
+      auto gpeer_identity = klass->is_valid(handler_, gacontext, gtoken, &error);
       g_bytes_unref(gtoken);
       g_object_unref(gacontext);
       if (gpeer_identity) {
         gsize gpeer_identity_size;
-        auto gpeer_identity_data = g_bytes_get_data(gpeer_identity,
-                                                    &gpeer_identity_size);
-        *peer_identity =
-          std::string(static_cast<const char *>(gpeer_identity_data),
-                      gpeer_identity_size);
+        auto gpeer_identity_data = g_bytes_get_data(gpeer_identity, &gpeer_identity_size);
+        *peer_identity = std::string(static_cast<const char *>(gpeer_identity_data),
+                                     gpeer_identity_size);
         g_bytes_unref(gpeer_identity);
       }
       if (error) {
@@ -622,7 +690,7 @@ namespace gaflight {
   private:
     GAFlightServerCustomAuthHandler *handler_;
   };
-}
+} // namespace gaflight
 G_BEGIN_DECLS
 
 G_DEFINE_TYPE(GAFlightServerCustomAuthHandler,
@@ -656,25 +724,20 @@ gaflight_server_custom_auth_handler_class_init(
  * Since: 12.0.0
  */
 void
-gaflight_server_custom_auth_handler_authenticate(
-  GAFlightServerCustomAuthHandler *handler,
-  GAFlightServerCallContext *context,
-  GAFlightServerAuthSender *sender,
-  GAFlightServerAuthReader *reader,
-  GError **error)
+gaflight_server_custom_auth_handler_authenticate(GAFlightServerCustomAuthHandler *handler,
+                                                 GAFlightServerCallContext *context,
+                                                 GAFlightServerAuthSender *sender,
+                                                 GAFlightServerAuthReader *reader,
+                                                 GError **error)
 {
   auto flight_handler =
-    gaflight_server_auth_handler_get_raw(
-      GAFLIGHT_SERVER_AUTH_HANDLER(handler));
+    gaflight_server_auth_handler_get_raw(GAFLIGHT_SERVER_AUTH_HANDLER(handler));
   auto flight_context = gaflight_server_call_context_get_raw(context);
   auto flight_sender = gaflight_server_auth_sender_get_raw(sender);
   auto flight_reader = gaflight_server_auth_reader_get_raw(reader);
-  auto status = flight_handler->Authenticate(*flight_context,
-                                             flight_sender,
-                                             flight_reader);
-  garrow::check(error,
-                status,
-                "[flight-server-custom-auth-handler][authenticate]");
+  auto status =
+    flight_handler->Authenticate(*flight_context, flight_sender, flight_reader);
+  garrow::check(error, status, "[flight-server-custom-auth-handler][authenticate]");
 }
 
 /**
@@ -683,44 +746,42 @@ gaflight_server_custom_auth_handler_authenticate(
  * @context: A #GAFlightServerCallContext.
  * @token: The client token. May be the empty string if the client does not
  *   provide a token.
- * @peer_identity: (out): The identity of the peer, if this authentication
- *   method supports it.
  * @error: (nullable): Return location for a #GError or %NULL.
  *
  * Validates a per-call client token.
  *
+ * Returns: (nullable) (transfer full): The identity of the peer, if
+ *   this authentication method supports it.
+ *
  * Since: 12.0.0
  */
-void
-gaflight_server_custom_auth_handler_is_valid(
-  GAFlightServerCustomAuthHandler *handler,
-  GAFlightServerCallContext *context,
-  GBytes *token,
-  GBytes **peer_identity,
-  GError **error)
+GBytes *
+gaflight_server_custom_auth_handler_is_valid(GAFlightServerCustomAuthHandler *handler,
+                                             GAFlightServerCallContext *context,
+                                             GBytes *token,
+                                             GError **error)
 {
   auto flight_handler =
-    gaflight_server_auth_handler_get_raw(
-      GAFLIGHT_SERVER_AUTH_HANDLER(handler));
+    gaflight_server_auth_handler_get_raw(GAFLIGHT_SERVER_AUTH_HANDLER(handler));
   gsize token_size;
   auto token_data = g_bytes_get_data(token, &token_size);
   auto flight_context = gaflight_server_call_context_get_raw(context);
   std::string flight_token(static_cast<const char *>(token_data), token_size);
   std::string flight_peer_identity;
-  auto status = flight_handler->IsValid(*flight_context,
-                                        flight_token,
-                                        &flight_peer_identity);
+  auto status =
+    flight_handler->IsValid(*flight_context, flight_token, &flight_peer_identity);
   if (garrow::check(error,
                     status,
                     "[flight-server-custom-auth-handler]"
                     "[is-valid]")) {
-    *peer_identity = g_bytes_new(flight_peer_identity.data(),
-                                 flight_peer_identity.size());
+    return g_bytes_new(flight_peer_identity.data(), flight_peer_identity.size());
+  } else {
+    return nullptr;
   }
 }
 
-
-typedef struct GAFlightServerOptionsPrivate_ {
+typedef struct GAFlightServerOptionsPrivate_
+{
   arrow::flight::FlightServerOptions options;
   GAFlightLocation *location;
   GAFlightServerAuthHandler *auth_handler;
@@ -731,14 +792,11 @@ enum {
   PROP_AUTH_HANDLER,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE(GAFlightServerOptions,
-                           gaflight_server_options,
-                           G_TYPE_OBJECT)
+G_DEFINE_TYPE_WITH_PRIVATE(GAFlightServerOptions, gaflight_server_options, G_TYPE_OBJECT)
 
-#define GAFLIGHT_SERVER_OPTIONS_GET_PRIVATE(obj)        \
-  static_cast<GAFlightServerOptionsPrivate *>(          \
-    gaflight_server_options_get_instance_private(       \
-      GAFLIGHT_SERVER_OPTIONS(obj)))
+#define GAFLIGHT_SERVER_OPTIONS_GET_PRIVATE(obj)                                         \
+  static_cast<GAFlightServerOptionsPrivate *>(                                           \
+    gaflight_server_options_get_instance_private(GAFLIGHT_SERVER_OPTIONS(obj)))
 
 static void
 gaflight_server_options_dispose(GObject *object)
@@ -781,7 +839,7 @@ gaflight_server_options_set_property(GObject *object,
     {
       priv->location = GAFLIGHT_LOCATION(g_value_dup_object(value));
       auto flight_location = gaflight_location_get_raw(priv->location);
-      new(&(priv->options)) arrow::flight::FlightServerOptions(*flight_location);
+      new (&(priv->options)) arrow::flight::FlightServerOptions(*flight_location);
     }
     break;
   case PROP_AUTH_HANDLER:
@@ -789,8 +847,7 @@ gaflight_server_options_set_property(GObject *object,
       if (priv->auth_handler) {
         g_object_unref(priv->auth_handler);
       }
-      priv->auth_handler =
-        GAFLIGHT_SERVER_AUTH_HANDLER(g_value_dup_object(value));
+      priv->auth_handler = GAFLIGHT_SERVER_AUTH_HANDLER(g_value_dup_object(value));
       if (priv->auth_handler) {
         priv->options.auth_handler =
           gaflight_server_auth_handler_get_raw(priv->auth_handler);
@@ -849,12 +906,12 @@ gaflight_server_options_class_init(GAFlightServerOptionsClass *klass)
    *
    * Since: 5.0.0
    */
-  spec = g_param_spec_object("location",
-                             "Location",
-                             "The location to be listened",
-                             GAFLIGHT_TYPE_LOCATION,
-                             static_cast<GParamFlags>(G_PARAM_READWRITE |
-                                                      G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_object(
+    "location",
+    "Location",
+    "The location to be listened",
+    GAFLIGHT_TYPE_LOCATION,
+    static_cast<GParamFlags>(G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_LOCATION, spec);
 
   /**
@@ -884,48 +941,37 @@ GAFlightServerOptions *
 gaflight_server_options_new(GAFlightLocation *location)
 {
   return static_cast<GAFlightServerOptions *>(
-    g_object_new(GAFLIGHT_TYPE_SERVER_OPTIONS,
-                 "location", location,
-                 NULL));
+    g_object_new(GAFLIGHT_TYPE_SERVER_OPTIONS, "location", location, NULL));
 }
 
-
-G_DEFINE_INTERFACE(GAFlightServable,
-                   gaflight_servable,
-                   G_TYPE_OBJECT)
+G_DEFINE_INTERFACE(GAFlightServable, gaflight_servable, G_TYPE_OBJECT)
 
 static void
 gaflight_servable_default_init(GAFlightServableInterface *iface)
 {
 }
 
-
 G_END_DECLS
 namespace gaflight {
   class Server : public arrow::flight::FlightServerBase {
   public:
-    explicit Server(GAFlightServer *gaserver) :
-      FlightServerBase(),
-      gaserver_(gaserver) {
-    }
+    explicit Server(GAFlightServer *gaserver) : FlightServerBase(), gaserver_(gaserver) {}
 
     ~Server() override = default;
 
     arrow::Status
-    ListFlights(
-      const arrow::flight::ServerCallContext &context,
-      const arrow::flight::Criteria *criteria,
-      std::unique_ptr<arrow::flight::FlightListing> *listing) override {
+    ListFlights(const arrow::flight::ServerCallContext &context,
+                const arrow::flight::Criteria *criteria,
+                std::unique_ptr<arrow::flight::FlightListing> *listing) override
+    {
       auto gacontext = gaflight_server_call_context_new_raw(&context);
       GAFlightCriteria *gacriteria = nullptr;
       if (criteria) {
         gacriteria = gaflight_criteria_new_raw(criteria);
       }
       GError *gerror = nullptr;
-      auto gaflights = gaflight_server_list_flights(gaserver_,
-                                                    gacontext,
-                                                    gacriteria,
-                                                    &gerror);
+      auto gaflights =
+        gaflight_server_list_flights(gaserver_, gacontext, gacriteria, &gerror);
       if (gacriteria) {
         g_object_unref(gacriteria);
       }
@@ -942,22 +988,20 @@ namespace gaflight {
         g_object_unref(gaflight);
       }
       g_list_free(gaflights);
-      *listing = std::make_unique<
-        arrow::flight::SimpleFlightListing>(flights);
+      *listing = std::make_unique<arrow::flight::SimpleFlightListing>(flights);
       return arrow::Status::OK();
     }
 
     arrow::Status
     GetFlightInfo(const arrow::flight::ServerCallContext &context,
                   const arrow::flight::FlightDescriptor &request,
-                  std::unique_ptr<arrow::flight::FlightInfo> *info) override {
+                  std::unique_ptr<arrow::flight::FlightInfo> *info) override
+    {
       auto gacontext = gaflight_server_call_context_new_raw(&context);
       auto garequest = gaflight_descriptor_new_raw(&request);
       GError *gerror = nullptr;
-      auto gainfo = gaflight_server_get_flight_info(gaserver_,
-                                                    gacontext,
-                                                    garequest,
-                                                    &gerror);
+      auto gainfo =
+        gaflight_server_get_flight_info(gaserver_, gacontext, garequest, &gerror);
       g_object_unref(garequest);
       g_object_unref(gacontext);
       if (gerror) {
@@ -965,23 +1009,20 @@ namespace gaflight {
                                       arrow::StatusCode::UnknownError,
                                       "[flight-server][get-flight-info]");
       }
-      *info = std::make_unique<arrow::flight::FlightInfo>(
-        *gaflight_info_get_raw(gainfo));
+      *info = std::make_unique<arrow::flight::FlightInfo>(*gaflight_info_get_raw(gainfo));
       g_object_unref(gainfo);
       return arrow::Status::OK();
     }
 
-    arrow::Status DoGet(
-      const arrow::flight::ServerCallContext &context,
-      const arrow::flight::Ticket &ticket,
-      std::unique_ptr<arrow::flight::FlightDataStream> *stream) override {
+    arrow::Status
+    DoGet(const arrow::flight::ServerCallContext &context,
+          const arrow::flight::Ticket &ticket,
+          std::unique_ptr<arrow::flight::FlightDataStream> *stream) override
+    {
       auto gacontext = gaflight_server_call_context_new_raw(&context);
       auto gaticket = gaflight_ticket_new_raw(&ticket);
       GError *gerror = nullptr;
-      auto gastream = gaflight_server_do_get(gaserver_,
-                                             gacontext,
-                                             gaticket,
-                                             &gerror);
+      auto gastream = gaflight_server_do_get(gaserver_, gacontext, gaticket, &gerror);
       g_object_unref(gaticket);
       g_object_unref(gacontext);
       if (gerror) {
@@ -996,10 +1037,11 @@ namespace gaflight {
   private:
     GAFlightServer *gaserver_;
   };
-};
+}; // namespace gaflight
 G_BEGIN_DECLS
 
-struct GAFlightServerPrivate {
+struct GAFlightServerPrivate
+{
   gaflight::Server server;
 };
 
@@ -1014,18 +1056,13 @@ gaflight_server_servable_interface_init(GAFlightServableInterface *iface)
   iface->get_raw = gaflight_server_servable_get_raw;
 }
 
-G_DEFINE_ABSTRACT_TYPE_WITH_CODE(GAFlightServer,
-                                 gaflight_server,
-                                 G_TYPE_OBJECT,
-                                 G_ADD_PRIVATE(GAFlightServer);
-                                 G_IMPLEMENT_INTERFACE(
-                                   GAFLIGHT_TYPE_SERVABLE,
-                                   gaflight_server_servable_interface_init))
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE(
+  GAFlightServer, gaflight_server, G_TYPE_OBJECT, G_ADD_PRIVATE(GAFlightServer);
+  G_IMPLEMENT_INTERFACE(GAFLIGHT_TYPE_SERVABLE, gaflight_server_servable_interface_init))
 
-#define GAFLIGHT_SERVER_GET_PRIVATE(object)         \
-  static_cast<GAFlightServerPrivate *>(             \
-    gaflight_server_get_instance_private(           \
-      GAFLIGHT_SERVER(object)))
+#define GAFLIGHT_SERVER_GET_PRIVATE(object)                                              \
+  static_cast<GAFlightServerPrivate *>(                                                  \
+    gaflight_server_get_instance_private(GAFLIGHT_SERVER(object)))
 
 G_END_DECLS
 static arrow::flight::FlightServerBase *
@@ -1049,7 +1086,7 @@ static void
 gaflight_server_init(GAFlightServer *object)
 {
   auto priv = GAFLIGHT_SERVER_GET_PRIVATE(object);
-  new(&(priv->server)) gaflight::Server(object);
+  new (&(priv->server)) gaflight::Server(object);
 }
 
 static void
@@ -1092,8 +1129,7 @@ gaflight_server_listen(GAFlightServer *server,
 gint
 gaflight_server_get_port(GAFlightServer *server)
 {
-  const auto flight_server =
-    gaflight_servable_get_raw(GAFLIGHT_SERVABLE(server));
+  const auto flight_server = gaflight_servable_get_raw(GAFLIGHT_SERVABLE(server));
   return flight_server->port();
 }
 
@@ -1110,13 +1146,10 @@ gaflight_server_get_port(GAFlightServer *server)
  * Since: 5.0.0
  */
 gboolean
-gaflight_server_shutdown(GAFlightServer *server,
-                         GError **error)
+gaflight_server_shutdown(GAFlightServer *server, GError **error)
 {
   auto flight_server = gaflight_servable_get_raw(GAFLIGHT_SERVABLE(server));
-  return garrow::check(error,
-                       flight_server->Shutdown(),
-                       "[flight-server][shutdown]");
+  return garrow::check(error, flight_server->Shutdown(), "[flight-server][shutdown]");
 }
 
 /**
@@ -1139,10 +1172,7 @@ gaflight_server_list_flights(GAFlightServer *server,
 {
   auto klass = GAFLIGHT_SERVER_GET_CLASS(server);
   if (!(klass && klass->list_flights)) {
-    g_set_error(error,
-                GARROW_ERROR,
-                GARROW_ERROR_NOT_IMPLEMENTED,
-                "not implemented");
+    g_set_error(error, GARROW_ERROR, GARROW_ERROR_NOT_IMPLEMENTED, "not implemented");
     return nullptr;
   }
   return (*(klass->list_flights))(server, context, criteria, error);
@@ -1167,10 +1197,7 @@ gaflight_server_get_flight_info(GAFlightServer *server,
 {
   auto klass = GAFLIGHT_SERVER_GET_CLASS(server);
   if (!(klass && klass->get_flight_info)) {
-    g_set_error(error,
-                GARROW_ERROR,
-                GARROW_ERROR_NOT_IMPLEMENTED,
-                "not implemented");
+    g_set_error(error, GARROW_ERROR, GARROW_ERROR_NOT_IMPLEMENTED, "not implemented");
     return nullptr;
   }
   return (*(klass->get_flight_info))(server, context, request, error);
@@ -1195,19 +1222,14 @@ gaflight_server_do_get(GAFlightServer *server,
 {
   auto klass = GAFLIGHT_SERVER_GET_CLASS(server);
   if (!(klass && klass->do_get)) {
-    g_set_error(error,
-                GARROW_ERROR,
-                GARROW_ERROR_NOT_IMPLEMENTED,
-                "not implemented");
+    g_set_error(error, GARROW_ERROR, GARROW_ERROR_NOT_IMPLEMENTED, "not implemented");
     return nullptr;
   }
   return (*(klass->do_get))(server, context, ticket, error);
 }
 
-
 G_END_DECLS
 
-
 arrow::flight::FlightDataStream *
 gaflight_data_stream_get_raw(GAFlightDataStream *stream)
 {
@@ -1215,14 +1237,34 @@ gaflight_data_stream_get_raw(GAFlightDataStream *stream)
   return priv->stream;
 }
 
+GAFlightMessageReader *
+gaflight_message_reader_new_raw(arrow::flight::FlightMessageReader *flight_reader,
+                                gboolean is_owner)
+{
+  return GAFLIGHT_MESSAGE_READER(g_object_new(GAFLIGHT_TYPE_MESSAGE_READER,
+                                              "reader",
+                                              flight_reader,
+                                              "is-owner",
+                                              is_owner,
+                                              NULL));
+}
+
+arrow::flight::FlightMessageReader *
+gaflight_message_reader_get_raw(GAFlightMessageReader *reader)
+{
+  auto flight_reader =
+    gaflight_record_batch_reader_get_raw(GAFLIGHT_RECORD_BATCH_READER(reader));
+  return static_cast<arrow::flight::FlightMessageReader *>(flight_reader);
+}
+
 GAFlightServerCallContext *
 gaflight_server_call_context_new_raw(
   const arrow::flight::ServerCallContext *flight_call_context)
 {
-  return GAFLIGHT_SERVER_CALL_CONTEXT(
-    g_object_new(GAFLIGHT_TYPE_SERVER_CALL_CONTEXT,
-                 "call-context", flight_call_context,
-                 NULL));
+  return GAFLIGHT_SERVER_CALL_CONTEXT(g_object_new(GAFLIGHT_TYPE_SERVER_CALL_CONTEXT,
+                                                   "call-context",
+                                                   flight_call_context,
+                                                   NULL));
 }
 
 const arrow::flight::ServerCallContext *
@@ -1233,13 +1275,10 @@ gaflight_server_call_context_get_raw(GAFlightServerCallContext *call_context)
 }
 
 GAFlightServerAuthSender *
-gaflight_server_auth_sender_new_raw(
-  arrow::flight::ServerAuthSender *flight_sender)
+gaflight_server_auth_sender_new_raw(arrow::flight::ServerAuthSender *flight_sender)
 {
   return GAFLIGHT_SERVER_AUTH_SENDER(
-    g_object_new(GAFLIGHT_TYPE_SERVER_AUTH_SENDER,
-                 "sender", flight_sender,
-                 nullptr));
+    g_object_new(GAFLIGHT_TYPE_SERVER_AUTH_SENDER, "sender", flight_sender, nullptr));
 }
 
 arrow::flight::ServerAuthSender *
@@ -1250,13 +1289,10 @@ gaflight_server_auth_sender_get_raw(GAFlightServerAuthSender *sender)
 }
 
 GAFlightServerAuthReader *
-gaflight_server_auth_reader_new_raw(
-  arrow::flight::ServerAuthReader *flight_reader)
+gaflight_server_auth_reader_new_raw(arrow::flight::ServerAuthReader *flight_reader)
 {
   return GAFLIGHT_SERVER_AUTH_READER(
-    g_object_new(GAFLIGHT_TYPE_SERVER_AUTH_READER,
-                 "reader", flight_reader,
-                 nullptr));
+    g_object_new(GAFLIGHT_TYPE_SERVER_AUTH_READER, "reader", flight_reader, nullptr));
 }
 
 arrow::flight::ServerAuthReader *
