@@ -17,14 +17,9 @@
  * under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <arrow/io/interfaces.h>
 #include <arrow/io/memory.h>
 #include <arrow/ipc/reader.h>
-#include <arrow/util/string_view.h>
 
 #include <arrow-glib/buffer.hpp>
 #include <arrow-glib/codec.hpp>
@@ -38,6 +33,23 @@
 #include <arrow-glib/tensor.hpp>
 
 #include <mutex>
+#include <string_view>
+
+static std::shared_ptr<arrow::io::FileInterface>
+garrow_input_stream_get_raw_file_interface(GArrowFile *file)
+{
+  auto input_stream = GARROW_INPUT_STREAM(file);
+  auto arrow_input_stream = garrow_input_stream_get_raw(input_stream);
+  return arrow_input_stream;
+}
+
+static std::shared_ptr<arrow::io::Readable>
+garrow_input_stream_get_raw_readable_interface(GArrowReadable *readable)
+{
+  auto input_stream = GARROW_INPUT_STREAM(readable);
+  auto arrow_input_stream = garrow_input_stream_get_raw(input_stream);
+  return arrow_input_stream;
+}
 
 G_BEGIN_DECLS
 
@@ -54,6 +66,8 @@ G_BEGIN_DECLS
  *
  * #GArrowBufferInputStream is a class to read data on buffer.
  *
+ * #GArrowFileInputStream is a class to read data in file.
+ *
  * #GArrowMemoryMappedInputStream is a class to read data in file by
  * mapping the file on memory. It supports zero copy.
  *
@@ -64,7 +78,8 @@ G_BEGIN_DECLS
  * compressed input stream.
  */
 
-typedef struct GArrowInputStreamPrivate_ {
+typedef struct GArrowInputStreamPrivate_
+{
   std::shared_ptr<arrow::io::InputStream> input_stream;
 } GArrowInputStreamPrivate;
 
@@ -72,48 +87,30 @@ enum {
   PROP_INPUT_STREAM = 1
 };
 
-static std::shared_ptr<arrow::io::FileInterface>
-garrow_input_stream_get_raw_file_interface(GArrowFile *file)
-{
-  auto input_stream = GARROW_INPUT_STREAM(file);
-  auto arrow_input_stream =
-    garrow_input_stream_get_raw(input_stream);
-  return arrow_input_stream;
-}
-
 static void
 garrow_input_stream_file_interface_init(GArrowFileInterface *iface)
 {
   iface->get_raw = garrow_input_stream_get_raw_file_interface;
 }
 
-static std::shared_ptr<arrow::io::Readable>
-garrow_input_stream_get_raw_readable_interface(GArrowReadable *readable)
-{
-  auto input_stream = GARROW_INPUT_STREAM(readable);
-  auto arrow_input_stream = garrow_input_stream_get_raw(input_stream);
-  return arrow_input_stream;
-}
-
 static void
 garrow_input_stream_readable_interface_init(GArrowReadableInterface *iface)
 {
   iface->get_raw = garrow_input_stream_get_raw_readable_interface;
 }
 
-G_DEFINE_TYPE_WITH_CODE(GArrowInputStream,
-                        garrow_input_stream,
-                        G_TYPE_INPUT_STREAM,
-                        G_ADD_PRIVATE(GArrowInputStream)
-                        G_IMPLEMENT_INTERFACE(GARROW_TYPE_FILE,
-                                              garrow_input_stream_file_interface_init)
-                        G_IMPLEMENT_INTERFACE(GARROW_TYPE_READABLE,
-                                              garrow_input_stream_readable_interface_init))
+G_DEFINE_TYPE_WITH_CODE(
+  GArrowInputStream,
+  garrow_input_stream,
+  G_TYPE_INPUT_STREAM,
+  G_ADD_PRIVATE(GArrowInputStream)
+    G_IMPLEMENT_INTERFACE(GARROW_TYPE_FILE, garrow_input_stream_file_interface_init)
+      G_IMPLEMENT_INTERFACE(GARROW_TYPE_READABLE,
+                            garrow_input_stream_readable_interface_init))
 
-#define GARROW_INPUT_STREAM_GET_PRIVATE(obj)         \
-  static_cast<GArrowInputStreamPrivate *>(           \
-     garrow_input_stream_get_instance_private(       \
-       GARROW_INPUT_STREAM(obj)))
+#define GARROW_INPUT_STREAM_GET_PRIVATE(obj)                                             \
+  static_cast<GArrowInputStreamPrivate *>(                                               \
+    garrow_input_stream_get_instance_private(GARROW_INPUT_STREAM(obj)))
 
 static void
 garrow_input_stream_finalize(GObject *object)
@@ -167,8 +164,7 @@ garrow_input_stream_read(GInputStream *stream,
   if (g_cancellable_set_error_if_cancelled(cancellable, error)) {
     return -1;
   }
-  auto arrow_input_stream =
-    garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream));
+  auto arrow_input_stream = garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream));
   auto n_read_bytes = arrow_input_stream->Read(count, buffer);
   if (!garrow::check(error, n_read_bytes, "[input-stream][read]")) {
     return -1;
@@ -185,8 +181,7 @@ garrow_input_stream_skip(GInputStream *stream,
   if (g_cancellable_set_error_if_cancelled(cancellable, error)) {
     return -1;
   }
-  auto arrow_input_stream =
-    garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream));
+  auto arrow_input_stream = garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream));
   auto status = arrow_input_stream->Advance(count);
   if (!garrow_error_check(error, status, "[input-stream][skip]")) {
     return -1;
@@ -195,15 +190,12 @@ garrow_input_stream_skip(GInputStream *stream,
 }
 
 static gboolean
-garrow_input_stream_close(GInputStream *stream,
-                          GCancellable *cancellable,
-                          GError **error)
+garrow_input_stream_close(GInputStream *stream, GCancellable *cancellable, GError **error)
 {
   if (g_cancellable_set_error_if_cancelled(cancellable, error)) {
     return FALSE;
   }
-  auto arrow_input_stream =
-    garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream));
+  auto arrow_input_stream = garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream));
   auto status = arrow_input_stream->Close();
   return garrow_error_check(error, status, "[input-stream][close]");
 }
@@ -212,14 +204,14 @@ static void
 garrow_input_stream_init(GArrowInputStream *object)
 {
   auto priv = GARROW_INPUT_STREAM_GET_PRIVATE(object);
-  new(&priv->input_stream) std::shared_ptr<arrow::io::InputStream>;
+  new (&priv->input_stream) std::shared_ptr<arrow::io::InputStream>;
 }
 
 static void
 garrow_input_stream_class_init(GArrowInputStreamClass *klass)
 {
   auto gobject_class = G_OBJECT_CLASS(klass);
-  gobject_class->finalize     = garrow_input_stream_finalize;
+  gobject_class->finalize = garrow_input_stream_finalize;
   gobject_class->set_property = garrow_input_stream_set_property;
   gobject_class->get_property = garrow_input_stream_get_property;
 
@@ -229,11 +221,11 @@ garrow_input_stream_class_init(GArrowInputStreamClass *klass)
   input_stream_class->close_fn = garrow_input_stream_close;
 
   GParamSpec *spec;
-  spec = g_param_spec_pointer("input-stream",
-                              "Input stream",
-                              "The raw std::shared<arrow::io::InputStream> *",
-                              static_cast<GParamFlags>(G_PARAM_WRITABLE |
-                                                       G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_pointer(
+    "input-stream",
+    "Input stream",
+    "The raw std::shared<arrow::io::InputStream> *",
+    static_cast<GParamFlags>(G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_INPUT_STREAM, spec);
 }
 
@@ -274,8 +266,7 @@ garrow_input_stream_align(GArrowInputStream *input_stream,
                           GError **error)
 {
   auto arrow_input_stream = garrow_input_stream_get_raw(input_stream);
-  auto status = arrow::ipc::AlignStream(arrow_input_stream.get(),
-                                        alignment);
+  auto status = arrow::ipc::AlignStream(arrow_input_stream.get(), alignment);
   return garrow_error_check(error, status, "[input-stream][align]");
 }
 
@@ -290,8 +281,7 @@ garrow_input_stream_align(GArrowInputStream *input_stream,
  * Since: 0.11.0
  */
 GArrowTensor *
-garrow_input_stream_read_tensor(GArrowInputStream *input_stream,
-                                GError **error)
+garrow_input_stream_read_tensor(GArrowInputStream *input_stream, GError **error)
 {
   auto arrow_input_stream = garrow_input_stream_get_raw(input_stream);
 
@@ -326,30 +316,23 @@ garrow_input_stream_read_record_batch(GArrowInputStream *input_stream,
 
   if (options) {
     auto arrow_options = garrow_read_options_get_raw(options);
-    auto arrow_dictionary_memo =
-      garrow_read_options_get_dictionary_memo_raw(options);
-    auto arrow_record_batch =
-      arrow::ipc::ReadRecordBatch(arrow_schema,
-                                  arrow_dictionary_memo,
-                                  *arrow_options,
-                                  arrow_input_stream.get());
-    if (garrow::check(error,
-                      arrow_record_batch,
-                      "[input-stream][read-record-batch]")) {
+    auto arrow_dictionary_memo = garrow_read_options_get_dictionary_memo_raw(options);
+    auto arrow_record_batch = arrow::ipc::ReadRecordBatch(arrow_schema,
+                                                          arrow_dictionary_memo,
+                                                          *arrow_options,
+                                                          arrow_input_stream.get());
+    if (garrow::check(error, arrow_record_batch, "[input-stream][read-record-batch]")) {
       return garrow_record_batch_new_raw(&(*arrow_record_batch));
     } else {
       return NULL;
     }
   } else {
     auto arrow_options = arrow::ipc::IpcReadOptions::Defaults();
-    auto arrow_record_batch =
-      arrow::ipc::ReadRecordBatch(arrow_schema,
-                                  nullptr,
-                                  arrow_options,
-                                  arrow_input_stream.get());
-    if (garrow::check(error,
-                      arrow_record_batch,
-                      "[input-stream][read-record-batch]")) {
+    auto arrow_record_batch = arrow::ipc::ReadRecordBatch(arrow_schema,
+                                                          nullptr,
+                                                          arrow_options,
+                                                          arrow_input_stream.get());
+    if (garrow::check(error, arrow_record_batch, "[input-stream][read-record-batch]")) {
       return garrow_record_batch_new_raw(&(*arrow_record_batch));
     } else {
       return NULL;
@@ -357,7 +340,6 @@ garrow_input_stream_read_record_batch(GArrowInputStream *input_stream,
   }
 }
 
-
 G_DEFINE_TYPE(GArrowSeekableInputStream,
               garrow_seekable_input_stream,
               GARROW_TYPE_INPUT_STREAM);
@@ -383,8 +365,7 @@ guint64
 garrow_seekable_input_stream_get_size(GArrowSeekableInputStream *input_stream,
                                       GError **error)
 {
-  auto arrow_random_access_file =
-    garrow_seekable_input_stream_get_raw(input_stream);
+  auto arrow_random_access_file = garrow_seekable_input_stream_get_raw(input_stream);
   auto size = arrow_random_access_file->GetSize();
   if (garrow::check(error, size, "[seekable-input-stream][get-size]")) {
     return size.ValueOrDie();
@@ -400,10 +381,10 @@ garrow_seekable_input_stream_get_size(GArrowSeekableInputStream *input_stream,
  * Returns: Whether zero copy read is supported or not.
  */
 gboolean
-garrow_seekable_input_stream_get_support_zero_copy(GArrowSeekableInputStream *input_stream)
+garrow_seekable_input_stream_get_support_zero_copy(
+  GArrowSeekableInputStream *input_stream)
 {
-  auto arrow_random_access_file =
-    garrow_seekable_input_stream_get_raw(input_stream);
+  auto arrow_random_access_file = garrow_seekable_input_stream_get_raw(input_stream);
   return arrow_random_access_file->supports_zero_copy();
 }
 
@@ -423,8 +404,7 @@ garrow_seekable_input_stream_read_at(GArrowSeekableInputStream *input_stream,
                                      gint64 n_bytes,
                                      GError **error)
 {
-  auto arrow_random_access_file =
-    garrow_seekable_input_stream_get_raw(input_stream);
+  auto arrow_random_access_file = garrow_seekable_input_stream_get_raw(input_stream);
 
   auto arrow_buffer = arrow_random_access_file->ReadAt(position, n_bytes);
   if (garrow::check(error, arrow_buffer, "[seekable-input-stream][read-at]")) {
@@ -452,8 +432,7 @@ garrow_seekable_input_stream_read_at_bytes(GArrowSeekableInputStream *input_stre
                                            gint64 n_bytes,
                                            GError **error)
 {
-  auto arrow_random_access_file =
-    garrow_seekable_input_stream_get_raw(input_stream);
+  auto arrow_random_access_file = garrow_seekable_input_stream_get_raw(input_stream);
 
   auto arrow_buffer_result = arrow_random_access_file->ReadAt(position, n_bytes);
   if (!garrow::check(error,
@@ -463,8 +442,7 @@ garrow_seekable_input_stream_read_at_bytes(GArrowSeekableInputStream *input_stre
   }
 
   auto arrow_cpu_buffer_result =
-    arrow::Buffer::ViewOrCopy(*arrow_buffer_result,
-                              arrow::default_cpu_memory_manager());
+    arrow::Buffer::ViewOrCopy(*arrow_buffer_result, arrow::default_cpu_memory_manager());
   if (!garrow::check(error,
                      arrow_cpu_buffer_result,
                      "[seekable-input-stream][read-at][bytes][view-or-copy]")) {
@@ -472,11 +450,9 @@ garrow_seekable_input_stream_read_at_bytes(GArrowSeekableInputStream *input_stre
   }
 
   auto arrow_cpu_buffer = *arrow_cpu_buffer_result;
-  return g_bytes_new(arrow_cpu_buffer->data(),
-                     arrow_cpu_buffer->size());
+  return g_bytes_new(arrow_cpu_buffer->data(), arrow_cpu_buffer->size());
 }
 
-
 /**
  * garrow_seekable_input_stream_peek:
  * @input_stream: A #GArrowSeekableInputStream.
@@ -496,8 +472,7 @@ garrow_seekable_input_stream_peek(GArrowSeekableInputStream *input_stream,
                                   gint64 n_bytes,
                                   GError **error)
 {
-  auto arrow_random_access_file =
-    garrow_seekable_input_stream_get_raw(input_stream);
+  auto arrow_random_access_file = garrow_seekable_input_stream_get_raw(input_stream);
 
   auto view_result = arrow_random_access_file->Peek(n_bytes);
   if (garrow::check(error, view_result, "[seekable-input-stream][peek]")) {
@@ -508,8 +483,8 @@ garrow_seekable_input_stream_peek(GArrowSeekableInputStream *input_stream,
   }
 }
 
-
-typedef struct GArrowBufferInputStreamPrivate_ {
+typedef struct GArrowBufferInputStreamPrivate_
+{
   GArrowBuffer *buffer;
 } GArrowBufferInputStreamPrivate;
 
@@ -521,10 +496,9 @@ G_DEFINE_TYPE_WITH_PRIVATE(GArrowBufferInputStream,
                            garrow_buffer_input_stream,
                            GARROW_TYPE_SEEKABLE_INPUT_STREAM);
 
-#define GARROW_BUFFER_INPUT_STREAM_GET_PRIVATE(obj)         \
-  static_cast<GArrowBufferInputStreamPrivate *>(            \
-     garrow_buffer_input_stream_get_instance_private(       \
-       GARROW_BUFFER_INPUT_STREAM(obj)))
+#define GARROW_BUFFER_INPUT_STREAM_GET_PRIVATE(obj)                                      \
+  static_cast<GArrowBufferInputStreamPrivate *>(                                         \
+    garrow_buffer_input_stream_get_instance_private(GARROW_BUFFER_INPUT_STREAM(obj)))
 
 static void
 garrow_buffer_input_stream_dispose(GObject *object)
@@ -585,17 +559,17 @@ garrow_buffer_input_stream_class_init(GArrowBufferInputStreamClass *klass)
 {
   auto gobject_class = G_OBJECT_CLASS(klass);
 
-  gobject_class->dispose      = garrow_buffer_input_stream_dispose;
+  gobject_class->dispose = garrow_buffer_input_stream_dispose;
   gobject_class->set_property = garrow_buffer_input_stream_set_property;
   gobject_class->get_property = garrow_buffer_input_stream_get_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_object("buffer",
-                             "Buffer",
-                             "The data",
-                             GARROW_TYPE_BUFFER,
-                             static_cast<GParamFlags>(G_PARAM_READWRITE |
-                                                      G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_object(
+    "buffer",
+    "Buffer",
+    "The data",
+    GARROW_TYPE_BUFFER,
+    static_cast<GParamFlags>(G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_BUFFER, spec);
 }
 
@@ -609,8 +583,7 @@ GArrowBufferInputStream *
 garrow_buffer_input_stream_new(GArrowBuffer *buffer)
 {
   auto arrow_buffer = garrow_buffer_get_raw(buffer);
-  auto arrow_buffer_reader =
-    std::make_shared<arrow::io::BufferReader>(arrow_buffer);
+  auto arrow_buffer_reader = std::make_shared<arrow::io::BufferReader>(arrow_buffer);
   return garrow_buffer_input_stream_new_raw(&arrow_buffer_reader, buffer);
 }
 
@@ -634,6 +607,81 @@ garrow_buffer_input_stream_get_buffer(GArrowBufferInputStream *input_stream)
   return garrow_buffer_new_raw(&arrow_buffer);
 }
 
+G_DEFINE_TYPE(GArrowFileInputStream,
+              garrow_file_input_stream,
+              GARROW_TYPE_SEEKABLE_INPUT_STREAM);
+
+static void
+garrow_file_input_stream_init(GArrowFileInputStream *object)
+{
+}
+
+static void
+garrow_file_input_stream_class_init(GArrowFileInputStreamClass *klass)
+{
+}
+
+/**
+ * garrow_file_input_stream_new:
+ * @path: The path of the file to be opened.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (nullable): A newly created #GArrowFileInputStream
+ *   or %NULL on error.
+ *
+ * Since: 6.0.0
+ */
+GArrowFileInputStream *
+garrow_file_input_stream_new(const gchar *path, GError **error)
+{
+  auto arrow_stream_result = arrow::io::ReadableFile::Open(path);
+  if (garrow::check(error, arrow_stream_result, "[file-input-stream][new]")) {
+    auto arrow_stream = *arrow_stream_result;
+    return garrow_file_input_stream_new_raw(&arrow_stream);
+  } else {
+    return NULL;
+  }
+}
+
+/**
+ * garrow_file_input_stream_new_file_descriptor:
+ * @file_descriptor: The file descriptor of this input stream.
+ * @error: (nullable): Return location for a #GError or %NULL.
+ *
+ * Returns: (nullable): A newly created #GArrowFileInputStream
+ *   or %NULL on error.
+ *
+ * Since: 6.0.0
+ */
+GArrowFileInputStream *
+garrow_file_input_stream_new_file_descriptor(gint file_descriptor, GError **error)
+{
+  auto arrow_stream_result = arrow::io::ReadableFile::Open(file_descriptor);
+  if (garrow::check(error,
+                    arrow_stream_result,
+                    "[file-input-stream][new-file-descriptor]")) {
+    auto arrow_stream = *arrow_stream_result;
+    return garrow_file_input_stream_new_raw(&arrow_stream);
+  } else {
+    return NULL;
+  }
+}
+
+/**
+ * garrow_file_input_stream_get_file_descriptor:
+ * @stream: A #GArrowFileInputStream.
+ *
+ * Returns: The file descriptor of @stream.
+ *
+ * Since: 6.0.0
+ */
+gint
+garrow_file_input_stream_get_file_descriptor(GArrowFileInputStream *stream)
+{
+  auto arrow_stream = std::static_pointer_cast<arrow::io::ReadableFile>(
+    garrow_input_stream_get_raw(GARROW_INPUT_STREAM(stream)));
+  return arrow_stream->file_descriptor();
+}
 
 G_DEFINE_TYPE(GArrowMemoryMappedInputStream,
               garrow_memory_mapped_input_stream,
@@ -658,50 +706,45 @@ garrow_memory_mapped_input_stream_class_init(GArrowMemoryMappedInputStreamClass
  *   or %NULL on error.
  */
 GArrowMemoryMappedInputStream *
-garrow_memory_mapped_input_stream_new(const gchar *path,
-                                      GError **error)
+garrow_memory_mapped_input_stream_new(const gchar *path, GError **error)
 {
-  auto arrow_memory_mapped_file_result =
-    arrow::io::MemoryMappedFile::Open(std::string(path),
-                                      arrow::io::FileMode::READ);
-  if (arrow_memory_mapped_file_result.ok()) {
-    auto arrow_memory_mapped_file =
-      arrow_memory_mapped_file_result.ValueOrDie();
-    return garrow_memory_mapped_input_stream_new_raw(&(arrow_memory_mapped_file));
+  auto arrow_stream_result =
+    arrow::io::MemoryMappedFile::Open(path, arrow::io::FileMode::READ);
+  if (garrow::check(error, arrow_stream_result, "[memory-mapped-input-stream][new]")) {
+    auto arrow_stream = *arrow_stream_result;
+    return garrow_memory_mapped_input_stream_new_raw(&arrow_stream);
   } else {
-    std::string context("[memory-mapped-input-stream][open]: <");
-    context += path;
-    context += ">";
-    garrow::check(error, arrow_memory_mapped_file_result, context.c_str());
     return NULL;
   }
 }
 
-
 G_END_DECLS
 
 namespace garrow {
   class GIOInputStream : public arrow::io::RandomAccessFile {
   public:
-    GIOInputStream(GInputStream *input_stream) :
-      input_stream_(input_stream),
-      lock_() {
+    GIOInputStream(GInputStream *input_stream) : input_stream_(input_stream), lock_()
+    {
       g_object_ref(input_stream_);
     }
 
-    ~GIOInputStream() {
-      g_object_unref(input_stream_);
-    }
+    ~GIOInputStream() { g_object_unref(input_stream_); }
 
-    GInputStream *get_input_stream() {
+    GInputStream *
+    get_input_stream()
+    {
       return input_stream_;
     }
 
-    bool closed() const override {
+    bool
+    closed() const override
+    {
       return static_cast<bool>(g_input_stream_is_closed(input_stream_));
     }
 
-    arrow::Status Close() override {
+    arrow::Status
+    Close() override
+    {
       std::lock_guard<std::mutex> guard(lock_);
       GError *error = NULL;
       if (g_input_stream_close(input_stream_, NULL, &error)) {
@@ -713,8 +756,11 @@ namespace garrow {
       }
     }
 
-    arrow::Result<int64_t> Tell() const override {
-      if (!G_IS_SEEKABLE(input_stream_)) {
+    arrow::Result<int64_t>
+    Tell() const override
+    {
+      if (!(G_IS_SEEKABLE(input_stream_) &&
+            g_seekable_can_seek(G_SEEKABLE(input_stream_)))) {
         std::string message("[gio-input-stream][tell] "
                             "not seekable input stream: <");
         message += G_OBJECT_CLASS_NAME(G_OBJECT_GET_CLASS(input_stream_));
@@ -725,58 +771,64 @@ namespace garrow {
       return g_seekable_tell(G_SEEKABLE(input_stream_));
     }
 
-    arrow::Result<int64_t> Read(int64_t n_bytes, void *out) override {
+    arrow::Result<int64_t>
+    Read(int64_t n_bytes, void *out) override
+    {
       std::lock_guard<std::mutex> guard(lock_);
       GError *error = NULL;
-      auto n_read_bytes = g_input_stream_read(input_stream_,
-                                              out,
-                                              n_bytes,
-                                              NULL,
-                                              &error);
-      if (n_read_bytes == -1) {
+      gsize n_read_bytes = 0;
+      auto success =
+        g_input_stream_read_all(input_stream_, out, n_bytes, &n_read_bytes, NULL, &error);
+      if (success) {
+        return n_read_bytes;
+      } else {
         return garrow_error_to_status(error,
                                       arrow::StatusCode::IOError,
                                       "[gio-input-stream][read]");
-      } else {
-        return n_read_bytes;
       }
     }
 
-    arrow::Result<int64_t> ReadAt(int64_t position,
-                                  int64_t n_bytes,
-                                  void* out) override {
+    arrow::Result<int64_t>
+    ReadAt(int64_t position, int64_t n_bytes, void *out) override
+    {
       return arrow::io::RandomAccessFile::ReadAt(position, n_bytes, out);
     }
 
     arrow::Result<std::shared_ptr<arrow::Buffer>>
-    ReadAt(int64_t position, int64_t n_bytes) override {
+    ReadAt(int64_t position, int64_t n_bytes) override
+    {
       return arrow::io::RandomAccessFile::ReadAt(position, n_bytes);
     }
 
     arrow::Result<std::shared_ptr<arrow::Buffer>>
-    Read(int64_t n_bytes) override {
+    Read(int64_t n_bytes) override
+    {
       ARROW_ASSIGN_OR_RAISE(auto buffer, arrow::AllocateResizableBuffer(n_bytes));
 
       std::lock_guard<std::mutex> guard(lock_);
       GError *error = NULL;
-      auto n_read_bytes = g_input_stream_read(input_stream_,
-                                              buffer->mutable_data(),
-                                              n_bytes,
-                                              NULL,
-                                              &error);
-      if (n_read_bytes == -1) {
-        return garrow_error_to_status(error,
-                                      arrow::StatusCode::IOError,
-                                      "[gio-input-stream][read][buffer]");
-      } else {
-        if (n_read_bytes < n_bytes) {
+      gsize n_read_bytes = 0;
+      auto success = g_input_stream_read_all(input_stream_,
+                                             buffer->mutable_data(),
+                                             n_bytes,
+                                             &n_read_bytes,
+                                             NULL,
+                                             &error);
+      if (success) {
+        if (n_read_bytes < static_cast<gsize>(n_bytes)) {
           RETURN_NOT_OK(buffer->Resize(n_read_bytes));
         }
         return std::move(buffer);
+      } else {
+        return garrow_error_to_status(error,
+                                      arrow::StatusCode::IOError,
+                                      "[gio-input-stream][read][buffer]");
       }
     }
 
-    arrow::Result<arrow::util::string_view> Peek(int64_t nbytes) override {
+    arrow::Result<std::string_view>
+    Peek(int64_t nbytes) override
+    {
       if (!G_IS_BUFFERED_INPUT_STREAM(input_stream_)) {
         std::string message("[gio-input-stream][peek] "
                             "not peekable input stream: <");
@@ -789,8 +841,7 @@ namespace garrow {
       auto available_n_bytes = g_buffered_input_stream_get_available(stream);
       if (available_n_bytes < static_cast<gsize>(nbytes)) {
         GError *error = NULL;
-        auto filled_size =
-          g_buffered_input_stream_fill(stream, nbytes, NULL, &error);
+        auto filled_size = g_buffered_input_stream_fill(stream, nbytes, NULL, &error);
         if (filled_size == -1) {
           return garrow_error_to_status(error,
                                         arrow::StatusCode::IOError,
@@ -803,11 +854,12 @@ namespace garrow {
       if (data_size > static_cast<gsize>(nbytes)) {
         data_size = nbytes;
       }
-      return arrow::util::string_view(static_cast<const char *>(data),
-                                      data_size);
+      return std::string_view(static_cast<const char *>(data), data_size);
     }
 
-    arrow::Status Seek(int64_t position) override {
+    arrow::Status
+    Seek(int64_t position) override
+    {
       if (!G_IS_SEEKABLE(input_stream_)) {
         std::string message("[gio-input-stream][seek] "
                             "not seekable input stream: <");
@@ -831,7 +883,9 @@ namespace garrow {
       }
     }
 
-    arrow::Result<int64_t> GetSize() override {
+    arrow::Result<int64_t>
+    GetSize() override
+    {
       if (!G_IS_SEEKABLE(input_stream_)) {
         std::string message("[gio-input-stream][size] "
                             "not seekable input stream: <");
@@ -843,11 +897,7 @@ namespace garrow {
       std::lock_guard<std::mutex> guard(lock_);
       auto current_position = g_seekable_tell(G_SEEKABLE(input_stream_));
       GError *error = NULL;
-      if (!g_seekable_seek(G_SEEKABLE(input_stream_),
-                           0,
-                           G_SEEK_END,
-                           NULL,
-                           &error)) {
+      if (!g_seekable_seek(G_SEEKABLE(input_stream_), 0, G_SEEK_END, NULL, &error)) {
         return garrow_error_to_status(error,
                                       arrow::StatusCode::IOError,
                                       "[gio-input-stream][size][seek]");
@@ -865,7 +915,9 @@ namespace garrow {
       return size;
     }
 
-    bool supports_zero_copy() const override {
+    bool
+    supports_zero_copy() const override
+    {
       return false;
     }
 
@@ -873,12 +925,12 @@ namespace garrow {
     GInputStream *input_stream_;
     std::mutex lock_;
   };
-};
+}; // namespace garrow
 
 G_BEGIN_DECLS
 
-
-typedef struct GArrowGIOInputStreamPrivate_ {
+typedef struct GArrowGIOInputStreamPrivate_
+{
   GInputStream *raw;
 } GArrowGIOInputStreamPrivate;
 
@@ -890,10 +942,9 @@ G_DEFINE_TYPE_WITH_PRIVATE(GArrowGIOInputStream,
                            garrow_gio_input_stream,
                            GARROW_TYPE_SEEKABLE_INPUT_STREAM);
 
-#define GARROW_GIO_INPUT_STREAM_GET_PRIVATE(object)     \
-  static_cast<GArrowGIOInputStreamPrivate *>(           \
-    garrow_gio_input_stream_get_instance_private(       \
-      GARROW_GIO_INPUT_STREAM(object)))
+#define GARROW_GIO_INPUT_STREAM_GET_PRIVATE(object)                                      \
+  static_cast<GArrowGIOInputStreamPrivate *>(                                            \
+    garrow_gio_input_stream_get_instance_private(GARROW_GIO_INPUT_STREAM(object)))
 
 static void
 garrow_gio_input_stream_dispose(GObject *object)
@@ -954,17 +1005,17 @@ garrow_gio_input_stream_class_init(GArrowGIOInputStreamClass *klass)
 {
   auto gobject_class = G_OBJECT_CLASS(klass);
 
-  gobject_class->dispose      = garrow_gio_input_stream_dispose;
+  gobject_class->dispose = garrow_gio_input_stream_dispose;
   gobject_class->set_property = garrow_gio_input_stream_set_property;
   gobject_class->get_property = garrow_gio_input_stream_get_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_object("raw",
-                             "Raw",
-                             "The raw GInputStream *",
-                             G_TYPE_INPUT_STREAM,
-                             static_cast<GParamFlags>(G_PARAM_READWRITE |
-                                                      G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_object(
+    "raw",
+    "Raw",
+    "The raw GInputStream *",
+    G_TYPE_INPUT_STREAM,
+    static_cast<GParamFlags>(G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_GIO_RAW, spec);
 }
 
@@ -979,11 +1030,12 @@ garrow_gio_input_stream_class_init(GArrowGIOInputStreamClass *klass)
 GArrowGIOInputStream *
 garrow_gio_input_stream_new(GInputStream *gio_input_stream)
 {
-  auto arrow_input_stream =
-    std::make_shared<garrow::GIOInputStream>(gio_input_stream);
+  auto arrow_input_stream = std::make_shared<garrow::GIOInputStream>(gio_input_stream);
   auto object = g_object_new(GARROW_TYPE_GIO_INPUT_STREAM,
-                             "input-stream", &arrow_input_stream,
-                             "raw", gio_input_stream,
+                             "input-stream",
+                             &arrow_input_stream,
+                             "raw",
+                             gio_input_stream,
                              NULL);
   auto input_stream = GARROW_GIO_INPUT_STREAM(object);
   return input_stream;
@@ -1006,7 +1058,8 @@ garrow_gio_input_stream_get_raw(GArrowGIOInputStream *input_stream)
   return priv->raw;
 }
 
-typedef struct GArrowCompressedInputStreamPrivate_ {
+typedef struct GArrowCompressedInputStreamPrivate_
+{
   GArrowCodec *codec;
   GArrowInputStream *raw;
 } GArrowCompressedInputStreamPrivate;
@@ -1020,9 +1073,9 @@ G_DEFINE_TYPE_WITH_PRIVATE(GArrowCompressedInputStream,
                            garrow_compressed_input_stream,
                            GARROW_TYPE_INPUT_STREAM)
 
-#define GARROW_COMPRESSED_INPUT_STREAM_GET_PRIVATE(object)      \
-  static_cast<GArrowCompressedInputStreamPrivate *>(            \
-    garrow_compressed_input_stream_get_instance_private(        \
+#define GARROW_COMPRESSED_INPUT_STREAM_GET_PRIVATE(object)                               \
+  static_cast<GArrowCompressedInputStreamPrivate *>(                                     \
+    garrow_compressed_input_stream_get_instance_private(                                 \
       GARROW_COMPRESSED_INPUT_STREAM(object)))
 
 static void
@@ -1095,25 +1148,25 @@ garrow_compressed_input_stream_class_init(GArrowCompressedInputStreamClass *klas
 {
   auto gobject_class = G_OBJECT_CLASS(klass);
 
-  gobject_class->dispose      = garrow_compressed_input_stream_dispose;
+  gobject_class->dispose = garrow_compressed_input_stream_dispose;
   gobject_class->set_property = garrow_compressed_input_stream_set_property;
   gobject_class->get_property = garrow_compressed_input_stream_get_property;
 
   GParamSpec *spec;
-  spec = g_param_spec_object("codec",
-                             "Codec",
-                             "The codec for the stream",
-                             GARROW_TYPE_CODEC,
-                             static_cast<GParamFlags>(G_PARAM_READWRITE |
-                                                      G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_object(
+    "codec",
+    "Codec",
+    "The codec for the stream",
+    GARROW_TYPE_CODEC,
+    static_cast<GParamFlags>(G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_CODEC, spec);
 
-  spec = g_param_spec_object("raw",
-                             "Raw",
-                             "The underlying raw input stream",
-                             GARROW_TYPE_INPUT_STREAM,
-                             static_cast<GParamFlags>(G_PARAM_READWRITE |
-                                                      G_PARAM_CONSTRUCT_ONLY));
+  spec = g_param_spec_object(
+    "raw",
+    "Raw",
+    "The underlying raw input stream",
+    GARROW_TYPE_INPUT_STREAM,
+    static_cast<GParamFlags>(G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property(gobject_class, PROP_RAW, spec);
 }
 
@@ -1134,8 +1187,7 @@ garrow_compressed_input_stream_new(GArrowCodec *codec,
 {
   auto arrow_codec = garrow_codec_get_raw(codec).get();
   auto arrow_raw = garrow_input_stream_get_raw(raw);
-  auto arrow_stream =
-    arrow::io::CompressedInputStream::Make(arrow_codec, arrow_raw);
+  auto arrow_stream = arrow::io::CompressedInputStream::Make(arrow_codec, arrow_raw);
   if (garrow::check(error, arrow_stream, "[compressed-input-stream][new]")) {
     return garrow_compressed_input_stream_new_raw(&(arrow_stream.ValueOrDie()),
                                                   codec,
@@ -1150,10 +1202,8 @@ G_END_DECLS
 GArrowInputStream *
 garrow_input_stream_new_raw(std::shared_ptr<arrow::io::InputStream> *arrow_input_stream)
 {
-  auto input_stream =
-    GARROW_INPUT_STREAM(g_object_new(GARROW_TYPE_INPUT_STREAM,
-                                     "input-stream", arrow_input_stream,
-                                     NULL));
+  auto input_stream = GARROW_INPUT_STREAM(
+    g_object_new(GARROW_TYPE_INPUT_STREAM, "input-stream", arrow_input_stream, NULL));
   return input_stream;
 }
 
@@ -1169,14 +1219,14 @@ garrow_seekable_input_stream_new_raw(
   std::shared_ptr<arrow::io::RandomAccessFile> *arrow_random_access_file)
 {
   auto object = g_object_new(GARROW_TYPE_SEEKABLE_INPUT_STREAM,
-                             "input-stream", arrow_random_access_file,
+                             "input-stream",
+                             arrow_random_access_file,
                              NULL);
   return GARROW_SEEKABLE_INPUT_STREAM(object);
 }
 
 std::shared_ptr<arrow::io::RandomAccessFile>
-garrow_seekable_input_stream_get_raw(
-  GArrowSeekableInputStream *seekable_input_stream)
+garrow_seekable_input_stream_get_raw(GArrowSeekableInputStream *seekable_input_stream)
 {
   auto arrow_input_stream =
     garrow_input_stream_get_raw(GARROW_INPUT_STREAM(seekable_input_stream));
@@ -1186,13 +1236,15 @@ garrow_seekable_input_stream_get_raw(
 }
 
 GArrowBufferInputStream *
-garrow_buffer_input_stream_new_raw(std::shared_ptr<arrow::io::BufferReader> *arrow_buffer_reader,
-                                   GArrowBuffer *buffer)
+garrow_buffer_input_stream_new_raw(
+  std::shared_ptr<arrow::io::BufferReader> *arrow_buffer_reader, GArrowBuffer *buffer)
 {
   auto buffer_input_stream =
     GARROW_BUFFER_INPUT_STREAM(g_object_new(GARROW_TYPE_BUFFER_INPUT_STREAM,
-                                            "input-stream", arrow_buffer_reader,
-                                            "buffer", buffer,
+                                            "input-stream",
+                                            arrow_buffer_reader,
+                                            "buffer",
+                                            buffer,
                                             NULL));
   return buffer_input_stream;
 }
@@ -1207,32 +1259,44 @@ garrow_buffer_input_stream_get_raw(GArrowBufferInputStream *buffer_input_stream)
   return arrow_buffer_reader;
 }
 
-GArrowMemoryMappedInputStream *
-garrow_memory_mapped_input_stream_new_raw(std::shared_ptr<arrow::io::MemoryMappedFile> *arrow_memory_mapped_file)
+GArrowFileInputStream *
+garrow_file_input_stream_new_raw(std::shared_ptr<arrow::io::ReadableFile> *arrow_stream)
 {
-  auto object = g_object_new(GARROW_TYPE_MEMORY_MAPPED_INPUT_STREAM,
-                             "input-stream", arrow_memory_mapped_file,
-                             NULL);
-  auto memory_mapped_input_stream = GARROW_MEMORY_MAPPED_INPUT_STREAM(object);
-  return memory_mapped_input_stream;
+  return GARROW_FILE_INPUT_STREAM(
+    g_object_new(GARROW_TYPE_FILE_INPUT_STREAM, "input-stream", arrow_stream, NULL));
+}
+
+GArrowMemoryMappedInputStream *
+garrow_memory_mapped_input_stream_new_raw(
+  std::shared_ptr<arrow::io::MemoryMappedFile> *arrow_stream)
+{
+  return GARROW_MEMORY_MAPPED_INPUT_STREAM(
+    g_object_new(GARROW_TYPE_MEMORY_MAPPED_INPUT_STREAM,
+                 "input-stream",
+                 arrow_stream,
+                 NULL));
 }
 
 GArrowCompressedInputStream *
-garrow_compressed_input_stream_new_raw(std::shared_ptr<arrow::io::CompressedInputStream> *arrow_raw,
-                                       GArrowCodec *codec,
-                                       GArrowInputStream *raw)
+garrow_compressed_input_stream_new_raw(
+  std::shared_ptr<arrow::io::CompressedInputStream> *arrow_raw,
+  GArrowCodec *codec,
+  GArrowInputStream *raw)
 {
-  auto compressed_input_stream =
-    g_object_new(GARROW_TYPE_COMPRESSED_INPUT_STREAM,
-                 "input-stream", arrow_raw,
-                 "codec", codec,
-                 "raw", raw,
-                 NULL);
+  auto compressed_input_stream = g_object_new(GARROW_TYPE_COMPRESSED_INPUT_STREAM,
+                                              "input-stream",
+                                              arrow_raw,
+                                              "codec",
+                                              codec,
+                                              "raw",
+                                              raw,
+                                              NULL);
   return GARROW_COMPRESSED_INPUT_STREAM(compressed_input_stream);
 }
 
 std::shared_ptr<arrow::io::InputStream>
-garrow_compressed_input_stream_get_raw(GArrowCompressedInputStream *compressed_input_stream)
+garrow_compressed_input_stream_get_raw(
+  GArrowCompressedInputStream *compressed_input_stream)
 {
   auto input_stream = GARROW_INPUT_STREAM(compressed_input_stream);
   auto arrow_input_stream = garrow_input_stream_get_raw(input_stream);
