@@ -25,11 +25,14 @@
 #include "arrow/status.h"
 #include "arrow/testing/gtest_util.h"
 #include "arrow/type.h"
+#include "arrow/util/decimal.h"
+#include "arrow/util/float16.h"
 #include "arrow/util/formatting.h"
 
 namespace arrow {
 
 using internal::StringFormatter;
+using util::Float16;
 
 class StringAppender {
  public:
@@ -279,6 +282,120 @@ TEST(Formatting, Double) {
   AssertFormatting(formatter, -HUGE_VAL, "-inf");
 }
 
+TEST(Formatting, HalfFloat) {
+  StringFormatter<HalfFloatType> formatter;
+
+  AssertFormatting(formatter, Float16(0.0f).bits(), "0");
+  AssertFormatting(formatter, Float16(-0.0f).bits(), "-0");
+  AssertFormatting(formatter, Float16(1.5f).bits(), "1.5");
+
+  // Slightly adapted from values present here
+  // https://blogs.mathworks.com/cleve/2017/05/08/half-precision-16-bit-floating-point-arithmetic/
+  AssertFormatting(formatter, 0x3c00, "1");
+  AssertFormatting(formatter, 0x3c01, "1.0009765625");
+  AssertFormatting(formatter, 0x0400, "0.00006103515625");
+  AssertFormatting(formatter, 0x0001, "5.960464477539063e-8");
+
+  // Can't avoid loss of precision here.
+  AssertFormatting(formatter, Float16(1234.567f).bits(), "1235");
+  AssertFormatting(formatter, Float16(1e3f).bits(), "1000");
+  AssertFormatting(formatter, Float16(1e4f).bits(), "10000");
+  AssertFormatting(formatter, Float16(1e10f).bits(), "inf");
+  AssertFormatting(formatter, Float16(1e15f).bits(), "inf");
+
+  AssertFormatting(formatter, 0xffff, "nan");
+  AssertFormatting(formatter, 0x7c00, "inf");
+  AssertFormatting(formatter, 0xfc00, "-inf");
+}
+
+template <typename T>
+void TestDecimalFormatter() {
+  struct TestParam {
+    int64_t test_value;
+    int32_t scale;
+    std::string expected_string;
+  };
+
+  // Borrow from Decimal::ToString test
+  const auto decimalTestData = std::vector<TestParam>{
+      {0, -1, "0.E+1"},
+      {0, 0, "0"},
+      {0, 1, "0.0"},
+      {0, 6, "0.000000"},
+      {2, 7, "2.E-7"},
+      {2, -1, "2.E+1"},
+      {2, 0, "2"},
+      {2, 1, "0.2"},
+      {2, 6, "0.000002"},
+      {-2, 7, "-2.E-7"},
+      {-2, 7, "-2.E-7"},
+      {-2, -1, "-2.E+1"},
+      {-2, 0, "-2"},
+      {-2, 1, "-0.2"},
+      {-2, 6, "-0.000002"},
+      {-2, 7, "-2.E-7"},
+      {123, -3, "1.23E+5"},
+      {123, -1, "1.23E+3"},
+      {123, 1, "12.3"},
+      {123, 0, "123"},
+      {123, 5, "0.00123"},
+      {123, 8, "0.00000123"},
+      {123, 9, "1.23E-7"},
+      {123, 10, "1.23E-8"},
+      {-123, -3, "-1.23E+5"},
+      {-123, -1, "-1.23E+3"},
+      {-123, 1, "-12.3"},
+      {-123, 0, "-123"},
+      {-123, 5, "-0.00123"},
+      {-123, 8, "-0.00000123"},
+      {-123, 9, "-1.23E-7"},
+      {-123, 10, "-1.23E-8"},
+      {1000000000, -3, "1.000000000E+12"},
+      {1000000000, -1, "1.000000000E+10"},
+      {1000000000, 0, "1000000000"},
+      {1000000000, 1, "100000000.0"},
+      {1000000000, 5, "10000.00000"},
+      {1000000000, 15, "0.000001000000000"},
+      {1000000000, 16, "1.000000000E-7"},
+      {1000000000, 17, "1.000000000E-8"},
+      {-1000000000, -3, "-1.000000000E+12"},
+      {-1000000000, -1, "-1.000000000E+10"},
+      {-1000000000, 0, "-1000000000"},
+      {-1000000000, 1, "-100000000.0"},
+      {-1000000000, 5, "-10000.00000"},
+      {-1000000000, 15, "-0.000001000000000"},
+      {-1000000000, 16, "-1.000000000E-7"},
+      {-1000000000, 17, "-1.000000000E-8"},
+      {1234567890123456789LL, -3, "1.234567890123456789E+21"},
+      {1234567890123456789LL, -1, "1.234567890123456789E+19"},
+      {1234567890123456789LL, 0, "1234567890123456789"},
+      {1234567890123456789LL, 1, "123456789012345678.9"},
+      {1234567890123456789LL, 5, "12345678901234.56789"},
+      {1234567890123456789LL, 24, "0.000001234567890123456789"},
+      {1234567890123456789LL, 25, "1.234567890123456789E-7"},
+      {-1234567890123456789LL, -3, "-1.234567890123456789E+21"},
+      {-1234567890123456789LL, -1, "-1.234567890123456789E+19"},
+      {-1234567890123456789LL, 0, "-1234567890123456789"},
+      {-1234567890123456789LL, 1, "-123456789012345678.9"},
+      {-1234567890123456789LL, 5, "-12345678901234.56789"},
+      {-1234567890123456789LL, 24, "-0.000001234567890123456789"},
+      {-1234567890123456789LL, 25, "-1.234567890123456789E-7"},
+  };
+
+  for (const auto& data : decimalTestData) {
+    const auto type = T(T::kMaxPrecision, data.scale);
+    StringFormatter<T> formatter(&type);
+    using value_type = typename TypeTraits<T>::CType;
+
+    AssertFormatting(formatter, value_type(data.test_value), data.expected_string);
+  }
+}
+
+TEST(Formatting, Decimals) {
+  TestDecimalFormatter<Decimal128Type>();
+  TestDecimalFormatter<Decimal256Type>();
+}
+
 TEST(Formatting, Date32) {
   StringFormatter<Date32Type> formatter;
 
@@ -433,6 +550,42 @@ TEST(Formatting, Timestamp) {
     AssertFormatting(formatter, -2203932304LL * 1000000000LL + 8,
                      "1900-02-28 12:34:56.000000008");
   }
+
+  {
+    auto timestamp_types = {timestamp(TimeUnit::SECOND, "US/Eastern"),
+                            timestamp(TimeUnit::SECOND, "+01:00")};
+    for (auto ty : timestamp_types) {
+      StringFormatter<TimestampType> formatter(ty.get());
+
+      AssertFormatting(formatter, 0, "1970-01-01 00:00:00Z");
+    }
+  }
+
+  {
+    constexpr int64_t kMillisInDay = 24 * 60 * 60 * 1000;
+    auto ty = timestamp(TimeUnit::MILLI, "+01:00");
+    StringFormatter<TimestampType> formatter(ty.get());
+    AssertFormatting(formatter, -15000 * 365 * kMillisInDay + 1,
+                     "-13021-12-17 00:00:00.001Z");
+  }
+
+  {
+    auto ty = timestamp(TimeUnit::MILLI, "Pacific/Maruesas");
+    StringFormatter<TimestampType> formatter(ty.get());
+    AssertFormatting(formatter, 0, "1970-01-01 00:00:00.000Z");
+  }
+
+  {
+    auto ty = timestamp(TimeUnit::MICRO, "-42:00");
+    StringFormatter<TimestampType> formatter(ty.get());
+    AssertFormatting(formatter, 0, "1970-01-01 00:00:00.000000Z");
+  }
+
+  {
+    auto ty = timestamp(TimeUnit::NANO, "Mars/Mariner_Valley");
+    StringFormatter<TimestampType> formatter(ty.get());
+    AssertFormatting(formatter, 0, "1970-01-01 00:00:00.000000000Z");
+  }
 }
 
 TEST(Formatting, Interval) {
