@@ -28,6 +28,7 @@ import uuid
 from io import StringIO
 from pathlib import Path
 from datetime import date
+import warnings
 
 import jinja2
 from ruamel.yaml import YAML
@@ -133,7 +134,7 @@ def _render_jinja_template(searchpath, template, params):
 
 # configurations for setting up branch skipping
 # - appveyor has a feature to skip builds without an appveyor.yml
-# - travis reads from the master branch and applies the rules
+# - travis reads from the default branch and applies the rules
 # - circle requires the configuration to be present on all branch, even ones
 #   that are configured to be skipped
 # - azure skips branches without azure-pipelines.yml by default
@@ -198,8 +199,8 @@ class GitRemoteCallbacks(PygitRemoteCallbacks):
             raise CrossbowError(msg)
 
         if (allowed_types &
-                pygit2.credentials.GIT_CREDENTIAL_USERPASS_PLAINTEXT):
-            return pygit2.UserPass(self.token, 'x-oauth-basic')
+                pygit2.credentials.CredentialType.USERPASS_PLAINTEXT):
+            return pygit2.UserPass('x-oauth-basic', self.token)
         else:
             return None
 
@@ -361,6 +362,27 @@ class Repo:
         return pygit2.Signature(self.user_name, self.user_email,
                                 int(time.time()))
 
+    @property
+    def default_branch_name(self):
+        default_branch_name = os.getenv("ARCHERY_DEFAULT_BRANCH")
+
+        if default_branch_name is None:
+            try:
+                ref_obj = self.repo.references["refs/remotes/origin/HEAD"]
+                target_name = ref_obj.target
+                target_name_tokenized = target_name.split("/")
+                default_branch_name = target_name_tokenized[-1]
+            except KeyError:
+                default_branch_name = "main"
+                warnings.warn('Unable to determine default branch name: '
+                              'ARCHERY_DEFAULT_BRANCH environment variable is '
+                              'not set. Git repository does not contain a '
+                              '\'refs/remotes/origin/HEAD\'reference. Setting '
+                              'the default branch name to ' +
+                              default_branch_name, RuntimeWarning)
+
+        return default_branch_name
+
     def create_tree(self, files):
         builder = self.repo.TreeBuilder()
 
@@ -382,7 +404,7 @@ class Repo:
         if parents is None:
             # by default use the main branch as the base of the new branch
             # required to reuse github actions cache across crossbow tasks
-            commit, _ = self.repo.resolve_refish("master")
+            commit, _ = self.repo.resolve_refish(self.default_branch_name)
             parents = [commit.id]
         tree_id = self.create_tree(files)
 
@@ -405,8 +427,14 @@ class Repo:
         return branch
 
     def create_tag(self, tag_name, commit_id, message=''):
+        git_object_commit = (
+            pygit2.GIT_OBJECT_COMMIT
+            if getattr(pygit2, 'GIT_OBJECT_COMMIT')
+            else pygit2.GIT_OBJ_COMMIT
+        )
         tag_id = self.repo.create_tag(tag_name, commit_id,
-                                      pygit2.GIT_OBJ_COMMIT, self.signature,
+                                      git_object_commit,
+                                      self.signature,
                                       message)
 
         # append to the pushable references
@@ -420,22 +448,39 @@ class Repo:
         blob = self.repo[entry.id]
         return blob.data
 
+    def _github_login(self, github_token):
+        """Returns a logged in github3.GitHub instance"""
+        if not _have_github3:
+            raise ImportError('Must install github3.py')
+        github_token = github_token or self.github_token
+        session = github3.session.GitHubSession(
+            default_connect_timeout=10,
+            default_read_timeout=30
+        )
+        github = github3.GitHub(session=session)
+        github.login(token=github_token)
+        return github
+
     def as_github_repo(self, github_token=None):
         """Converts it to a repository object which wraps the GitHub API"""
         if self._github_repo is None:
-            if not _have_github3:
-                raise ImportError('Must install github3.py')
-            github_token = github_token or self.github_token
+            github = self._github_login(github_token)
             username, reponame = _parse_github_user_repo(self.remote_url)
-            session = github3.session.GitHubSession(
-                default_connect_timeout=10,
-                default_read_timeout=30
-            )
-            github = github3.GitHub(session=session)
-            github.login(token=github_token)
             self._github_repo = github.repository(username, reponame)
         return self._github_repo
 
+    def token_expiration_date(self, github_token=None):
+        """Returns the expiration date for the github_token provided"""
+        github = self._github_login(github_token)
+        # github3 hides the headers from us. Use the _get method
+        # to access the response headers.
+        resp = github._get(github.session.base_url)
+        # Response in the form '2023-01-23 10:40:28 UTC'
+        date_string = resp.headers.get(
+            'github-authentication-token-expiration')
+        if date_string:
+            return date.fromisoformat(date_string.split()[0])
+
     def github_commit(self, sha):
         repo = self.as_github_repo()
         return repo.commit(sha)
@@ -463,7 +508,7 @@ class Repo:
                 logger.error('Attempt {} has failed with message: {}.'
                              .format(i + 1, str(e)))
                 logger.error('Error message {}'.format(e.msg))
-                logger.error('List of errors provided by Github:')
+                logger.error('List of errors provided by GitHub:')
                 for err in e.errors:
                     logger.error(' - {}'.format(err))
 
@@ -487,7 +532,7 @@ class Repo:
 
             time.sleep(retry_backoff)
 
-        raise RuntimeError('Github asset uploading has failed!')
+        raise RuntimeError('GitHub asset uploading has failed!')
 
     def github_upload_asset_curl(self, release, path, name, mime):
         upload_url, _ = release.upload_url.split('{?')
@@ -546,8 +591,11 @@ class Repo:
                         'Unsupported upload method {}'.format(method)
                     )
 
-    def github_pr(self, title, head=None, base="master", body=None,
+    def github_pr(self, title, head=None, base=None, body=None,
                   github_token=None, create=False):
+        if create:
+            # Default value for base is the default_branch_name
+            base = self.default_branch_name if base is None else base
         github_token = github_token or self.github_token
         repo = self.as_github_repo(github_token=github_token)
         if create:
@@ -748,9 +796,6 @@ class Target(Serializable):
         self.r_version = r_version
         self.no_rc_version = re.sub(r'-rc\d+\Z', '', version)
         self.no_rc_r_version = re.sub(r'-rc\d+\Z', '', r_version)
-        # TODO(ARROW-17552): Remove "master" from default_branch after
-        #                    migration to "main".
-        self.default_branch = ['main', 'master']
         # Semantic Versioning 1.0.0: https://semver.org/spec/v1.0.0.html
         #
         # > A pre-release version number MAY be denoted by appending an
@@ -828,9 +873,7 @@ class Target(Serializable):
                    version=version, r_version=r_version)
 
     def is_default_branch(self):
-        # TODO(ARROW-17552): Switch the condition to "is" instead of "in"
-        #                    once "master" is removed from "default_branch".
-        return self.branch in self.default_branch
+        return self.branch == 'main'
 
 
 class Task(Serializable):
@@ -992,8 +1035,8 @@ class TaskAssets(dict):
 
     def __init__(self, github_release, artifact_patterns,
                  validate_patterns=True):
-        # HACK(kszucs): don't expect uploaded assets of no atifacts were
-        # defiened for the tasks in order to spare a bit of github rate limit
+        # HACK(kszucs): don't expect uploaded assets of no artifacts were
+        # defined for the tasks in order to spare a bit of github rate limit
         if not artifact_patterns:
             return
 
@@ -1105,7 +1148,7 @@ class Job(Serializable):
     @classmethod
     def from_config(cls, config, target, tasks=None, groups=None, params=None):
         """
-        Intantiate a job from based on a config.
+        Instantiate a job from based on a config.
 
         Parameters
         ----------
@@ -1265,7 +1308,7 @@ class Config(dict):
         }
 
     def validate(self):
-        # validate that the task groups are properly refering to the tasks
+        # validate that the task groups are properly referring to the tasks
         for group_name, group in self['groups'].items():
             for pattern in group:
                 # remove the negation character for blocklisted tasks
@@ -1289,11 +1332,15 @@ class Config(dict):
                     'is: `{}`'.format(task_name, str(e))
                 )
 
+        # Get the default branch name from the repository
+        arrow_source_dir = ArrowSources.find()
+        repo = Repo(arrow_source_dir.path)
+
         # validate that the defined tasks are renderable, in order to to that
         # define the required object with dummy data
         target = Target(
             head='e279a7e06e61c14868ca7d71dea795420aea6539',
-            branch='master',
+            branch=repo.default_branch_name,
             remote='https://github.com/apache/arrow',
             version='1.0.0dev123',
             r_version='0.13.0.100000123',
