@@ -20,11 +20,11 @@
 
 # Requirements
 # - Ruby >= 2.3
-# - Maven >= 3.3.9
-# - JDK >=7
+# - Maven >= 3.8.7
+# - JDK >=8
 # - gcc >= 4.8
-# - Node.js >= 11.12 (best way is to use nvm)
-# - Go >= 1.15
+# - Node.js >= 18
+# - Go >= 1.21
 # - Docker
 #
 # If using a non-system Boost, set BOOST_ROOT and add Boost libraries to
@@ -153,17 +153,17 @@ verify_dir_artifact_signatures() {
   # verify the signature and the checksums of each artifact
   find $1 -name '*.asc' | while read sigfile; do
     artifact=${sigfile/.asc/}
-    gpg --verify $sigfile $artifact || exit 1
+    gpg --verify $sigfile $artifact
 
     # go into the directory because the checksum files contain only the
     # basename of the artifact
     pushd $(dirname $artifact)
     base_artifact=$(basename $artifact)
     if [ -f $base_artifact.sha256 ]; then
-      ${sha256_verify} $base_artifact.sha256 || exit 1
+      ${sha256_verify} $base_artifact.sha256
     fi
     if [ -f $base_artifact.sha512 ]; then
-      ${sha512_verify} $base_artifact.sha512 || exit 1
+      ${sha512_verify} $base_artifact.sha512
     fi
     popd
   done
@@ -171,9 +171,9 @@ verify_dir_artifact_signatures() {
 
 test_binary() {
   show_header "Testing binary artifacts"
-  maybe_setup_conda || exit 1
+  maybe_setup_conda
 
-  local download_dir=binaries
+  local download_dir=${ARROW_TMPDIR}/binaries
   mkdir -p ${download_dir}
 
   ${PYTHON:-python3} $SOURCE_DIR/download_rc_binaries.py $VERSION $RC_NUMBER \
@@ -185,27 +185,28 @@ test_binary() {
 test_apt() {
   show_header "Testing APT packages"
 
-  for target in "debian:bullseye" \
-                "arm64v8/debian:bullseye" \
-                "debian:bookworm" \
+  for target in "debian:bookworm" \
                 "arm64v8/debian:bookworm" \
-                "ubuntu:bionic" \
-                "arm64v8/ubuntu:bionic" \
+                "debian:trixie" \
+                "arm64v8/debian:trixie" \
                 "ubuntu:focal" \
                 "arm64v8/ubuntu:focal" \
                 "ubuntu:jammy" \
-                "arm64v8/ubuntu:jammy"; do \
+                "arm64v8/ubuntu:jammy" \
+                "ubuntu:noble" \
+                "arm64v8/ubuntu:noble"; do \
     case "${target}" in
       arm64v8/*)
         if [ "$(arch)" = "aarch64" -o -e /usr/bin/qemu-aarch64-static ]; then
           case "${target}" in
-          arm64v8/debian:buster|arm64v8/ubuntu:bionic|arm64v8/ubuntu:focal)
-            ;; # OK
-          *)
-            # qemu-user-static in Ubuntu 20.04 has a crash bug:
-            #   https://bugs.launchpad.net/qemu/+bug/1749393
-            continue
-            ;;
+            arm64v8/ubuntu:focal)
+              : # OK
+              ;;
+            *)
+              # qemu-user-static in Ubuntu 20.04 has a crash bug:
+              #   https://bugs.launchpad.net/qemu/+bug/1749393
+              continue
+              ;;
           esac
         else
           continue
@@ -213,6 +214,7 @@ test_apt() {
         ;;
     esac
     if ! docker run --rm -v "${ARROW_DIR}":/arrow:delegated \
+           --security-opt="seccomp=unconfined" \
            "${target}" \
            /arrow/dev/release/verify-apt.sh \
            "${VERSION}" \
@@ -230,7 +232,7 @@ test_yum() {
                 "arm64v8/almalinux:9" \
                 "almalinux:8" \
                 "arm64v8/almalinux:8" \
-                "amazonlinux:2" \
+                "amazonlinux:2023" \
                 "quay.io/centos/centos:stream9" \
                 "quay.io/centos/centos:stream8" \
                 "centos:7"; do
@@ -313,15 +315,17 @@ install_nodejs() {
     return 0
   fi
 
-  required_node_major_version=16
   node_major_version=$(node --version 2>&1 | grep -o '^v[0-9]*' | sed -e 's/^v//g' || :)
-
-  if [ -n "${node_major_version}" ] && [ "${node_major_version}" -ge ${required_node_major_version} ]; then
-    show_info "Found NodeJS installation with major version ${node_major_version}"
+  node_minor_version=$(node --version 2>&1 | grep -o '^v[0-9]*\.[0-9]*' | sed -e 's/^v[0-9]*\.//g' || :)
+  if [[ -n "${node_major_version}" && -n "${node_minor_version}" &&
+      ("${node_major_version}" -eq 16 ||
+        ("${node_major_version}" -eq 18 && "${node_minor_version}" -ge 14) ||
+        "${node_major_version}" -ge 20) ]]; then
+    show_info "Found NodeJS installation with version v${node_major_version}.${node_minor_version}.x"
   else
-    export NVM_DIR="`pwd`/.nvm"
+    export NVM_DIR="$(pwd)/.nvm"
     mkdir -p $NVM_DIR
-    curl -sL https://raw.githubusercontent.com/nvm-sh/nvm/v0.39.0/install.sh | \
+    curl -sL https://raw.githubusercontent.com/nvm-sh/nvm/v0.39.3/install.sh | \
       PROFILE=/dev/null bash
     [ -s "$NVM_DIR/nvm.sh" ] && \. "$NVM_DIR/nvm.sh"
 
@@ -341,18 +345,15 @@ install_csharp() {
 
   show_info "Ensuring that C# is installed..."
 
-  if which dotnet > /dev/null 2>&1; then
+  if dotnet --version | grep 8\.0 > /dev/null 2>&1; then
     local csharp_bin=$(dirname $(which dotnet))
-    if ! which sourcelink > /dev/null 2>&1; then
-      local dotnet_tools_dir=$HOME/.dotnet/tools
-      if [ -d "${dotnet_tools_dir}" ]; then
-        PATH="${dotnet_tools_dir}:$PATH"
-      fi
-    fi
     show_info "Found C# at $(which csharp) (.NET $(dotnet --version))"
   else
+    if which dotnet > /dev/null 2>&1; then
+      show_info "dotnet found but it is the wrong version and will be ignored."
+    fi
     local csharp_bin=${ARROW_TMPDIR}/csharp/bin
-    local dotnet_version=6.0.202
+    local dotnet_version=8.0.204
     local dotnet_platform=
     case "$(uname)" in
       Linux)
@@ -365,8 +366,8 @@ install_csharp() {
     local dotnet_download_thank_you_url=https://dotnet.microsoft.com/download/thank-you/dotnet-sdk-${dotnet_version}-${dotnet_platform}-x64-binaries
     local dotnet_download_url=$( \
       curl -sL ${dotnet_download_thank_you_url} | \
-        grep 'window\.open' | \
-        grep -E -o '[^"]+' | \
+        grep 'directLink' | \
+        grep -E -o 'https://download[^"]+' | \
         sed -n 2p)
     mkdir -p ${csharp_bin}
     curl -sL ${dotnet_download_url} | \
@@ -376,10 +377,11 @@ install_csharp() {
   fi
 
   # Ensure to have sourcelink installed
-  if ! which sourcelink > /dev/null 2>&1; then
-    dotnet tool install --tool-path ${csharp_bin} sourcelink
+  if ! dotnet tool list | grep sourcelink > /dev/null 2>&1; then
+    dotnet new tool-manifest
+    dotnet tool install --local sourcelink
     PATH=${csharp_bin}:${PATH}
-    if ! sourcelink --help > /dev/null 2>&1; then
+    if ! dotnet tool run sourcelink --help > /dev/null 2>&1; then
       export DOTNET_ROOT=${csharp_bin}
     fi
   fi
@@ -396,10 +398,12 @@ install_go() {
 
   if command -v go > /dev/null; then
     show_info "Found $(go version) at $(command -v go)"
+    export GOPATH=${ARROW_TMPDIR}/gopath
+    mkdir -p $GOPATH
     return 0
   fi
 
-  local version=1.17.13
+  local version=1.21.8
   show_info "Installing go version ${version}..."
 
   local arch="$(uname -m)"
@@ -416,8 +420,9 @@ install_go() {
   fi
 
   local archive="go${version}.${os}-${arch}.tar.gz"
-  curl -sLO https://dl.google.com/go/$archive
+  curl -sLO https://go.dev/dl/$archive
 
+  ls -l
   local prefix=${ARROW_TMPDIR}/go
   mkdir -p $prefix
   tar -xzf $archive -C $prefix
@@ -427,6 +432,7 @@ install_go() {
   export GOPATH=${prefix}/gopath
   export PATH=$GOROOT/bin:$GOPATH/bin:$PATH
 
+  mkdir -p $GOPATH
   show_info "$(go version) installed at $(which go)"
 
   GO_ALREADY_INSTALLED=1
@@ -491,6 +497,46 @@ maybe_setup_conda() {
   fi
 }
 
+install_maven() {
+  MAVEN_VERSION=3.8.7
+  if command -v mvn > /dev/null; then
+    # --batch-mode is for disabling output color.
+    SYSTEM_MAVEN_VERSION=$(mvn --batch-mode -v | head -n 1 | awk '{print $3}')
+    show_info "Found Maven version ${SYSTEM_MAVEN_VERSION} at $(command -v mvn)."
+  else
+    SYSTEM_MAVEN_VERSION=0.0.0
+    show_info "Maven installation not found."
+  fi
+
+  if [[ "$MAVEN_VERSION" == "$SYSTEM_MAVEN_VERSION" ]]; then
+    show_info "System Maven version ${SYSTEM_MAVEN_VERSION} matches required Maven version ${MAVEN_VERSION}. Skipping installation."
+  else
+    # Append pipe character to make preview release versions like "X.Y.Z-beta-1" sort
+    # as older than their corresponding release version "X.Y.Z". This works because 
+    # `sort -V` orders the pipe character lower than any version number character.
+    older_version=$(printf '%s\n%s\n' "$SYSTEM_MAVEN_VERSION" "$MAVEN_VERSION" | sed 's/$/|/' | sort -V | sed 's/|$//' | head -n1)
+    if [[ "$older_version" == "$SYSTEM_MAVEN_VERSION" ]]; then
+      show_info "Installing Maven version ${MAVEN_VERSION}..."
+      APACHE_MIRROR="https://www.apache.org/dyn/closer.lua?action=download&filename="
+      curl -sL -o apache-maven-${MAVEN_VERSION}-bin.tar.gz \
+        ${APACHE_MIRROR}/maven/maven-3/${MAVEN_VERSION}/binaries/apache-maven-${MAVEN_VERSION}-bin.tar.gz
+      tar xzf apache-maven-${MAVEN_VERSION}-bin.tar.gz
+      export PATH=$(pwd)/apache-maven-${MAVEN_VERSION}/bin:$PATH
+      # --batch-mode is for disabling output color.
+      show_info "Installed Maven version $(mvn --batch-mode -v | head -n 1 | awk '{print $3}')"
+    else
+      show_info "System Maven version ${SYSTEM_MAVEN_VERSION} is newer than minimum version ${MAVEN_VERSION}. Skipping installation."
+    fi
+  fi
+}
+
+maybe_setup_maven() {
+  show_info "Ensuring that Maven is installed..."
+  if [ "${USE_CONDA}" -eq 0 ]; then
+    install_maven
+  fi
+}
+
 maybe_setup_virtualenv() {
   # Optionally setup pip virtualenv with the passed dependencies
   local env="venv-${VENV_ENV:-source}"
@@ -557,12 +603,59 @@ maybe_setup_nodejs() {
 test_package_java() {
   show_header "Build and test Java libraries"
 
-  # Build and test Java (Requires newer Maven -- I used 3.3.9)
-  maybe_setup_conda maven || exit 1
+  maybe_setup_maven
+  maybe_setup_conda maven openjdk
 
   pushd java
-  mvn test
+
+  if [ ${TEST_INTEGRATION_JAVA} -gt 0 ]; then
+    # Build JNI for C data interface
+    local -a cmake_options=()
+    # Enable only C data interface.
+    cmake_options+=(-DARROW_JAVA_JNI_ENABLE_C=ON)
+    cmake_options+=(-DARROW_JAVA_JNI_ENABLE_DEFAULT=OFF)
+    # Disable Testing because GTest might not be present.
+    cmake_options+=(-DBUILD_TESTING=OFF)
+    if [ ! -z "${CMAKE_GENERATOR}" ]; then
+      cmake_options+=(-G "${CMAKE_GENERATOR}")
+    fi
+    local build_dir="${ARROW_TMPDIR}/java-jni-build"
+    local install_dir="${ARROW_TMPDIR}/java-jni-install"
+    local dist_dir="${ARROW_TMPDIR}/java-jni-dist"
+    cmake \
+      -S . \
+      -B "${build_dir}" \
+      -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE:-release} \
+      -DCMAKE_INSTALL_LIBDIR=lib \
+      -DCMAKE_INSTALL_PREFIX="${install_dir}" \
+      -DCMAKE_PREFIX_PATH="${ARROW_HOME}" \
+      "${cmake_options[@]}"
+    cmake --build "${build_dir}"
+    cmake --install "${build_dir}"
+
+    local normalized_arch=$(arch)
+    case ${normalized_arch} in
+      aarch64|arm64)
+        normalized_arch=aarch_64
+        ;;
+      i386)
+        normalized_arch=x86_64
+        ;;
+    esac
+    mkdir -p ${dist_dir}
+    mv ${install_dir}/lib/* ${dist_dir}
+    mvn install \
+        -Darrow.c.jni.dist.dir=${dist_dir} \
+        -Parrow-c-data
+  fi
+
+  if [ ${TEST_JAVA} -gt 0 ]; then
+    mvn test
+  fi
+
+  # Build jars
   mvn package
+
   popd
 }
 
@@ -570,7 +663,7 @@ test_and_install_cpp() {
   show_header "Build, install and test C++ libraries"
 
   # Build and test C++
-  maybe_setup_virtualenv numpy || exit 1
+  maybe_setup_virtualenv numpy
   maybe_setup_conda \
     --file ci/conda_env_unix.txt \
     --file ci/conda_env_cpp.txt \
@@ -578,7 +671,7 @@ test_and_install_cpp() {
     ncurses \
     numpy \
     sqlite \
-    compilers || exit 1
+    compilers
 
   if [ "${USE_CONDA}" -gt 0 ]; then
     DEFAULT_DEPENDENCY_SOURCE="CONDA"
@@ -594,11 +687,20 @@ test_and_install_cpp() {
     ARROW_CMAKE_OPTIONS="${ARROW_CMAKE_OPTIONS:-} -G ${CMAKE_GENERATOR}"
   fi
 
+  local ARROW_BUILD_INTEGRATION=OFF
+  local ARROW_BUILD_TESTS=OFF
+  if [ ${TEST_INTEGRATION_CPP} -gt 0 ]; then
+    ARROW_BUILD_INTEGRATION=ON
+  fi
+  if [ ${TEST_CPP} -gt 0 ]; then
+    ARROW_BUILD_TESTS=ON
+  fi
+
   cmake \
     -DARROW_BOOST_USE_SHARED=ON \
     -DARROW_BUILD_EXAMPLES=OFF \
-    -DARROW_BUILD_INTEGRATION=ON \
-    -DARROW_BUILD_TESTS=ON \
+    -DARROW_BUILD_INTEGRATION=${ARROW_BUILD_INTEGRATION} \
+    -DARROW_BUILD_TESTS=${ARROW_BUILD_TESTS} \
     -DARROW_BUILD_UTILITIES=ON \
     -DARROW_COMPUTE=ON \
     -DARROW_CSV=ON \
@@ -614,7 +716,7 @@ test_and_install_cpp() {
     -DARROW_JSON=ON \
     -DARROW_ORC=ON \
     -DARROW_PARQUET=ON \
-    -DARROW_PLASMA=${ARROW_PLASMA} \
+    -DARROW_SUBSTRAIT=ON \
     -DARROW_S3=${ARROW_S3} \
     -DARROW_USE_CCACHE=${ARROW_USE_CCACHE:-ON} \
     -DARROW_VERBOSE_THIRDPARTY_BUILD=ON \
@@ -639,16 +741,13 @@ test_and_install_cpp() {
   export CMAKE_BUILD_PARALLEL_LEVEL=${CMAKE_BUILD_PARALLEL_LEVEL:-${NPROC}}
   cmake --build . --target install
 
-  # Explicitly set site-package directory, otherwise the C++ tests are unable
-  # to load numpy in a python virtualenv
-  local pythonpath=$(python -c "import site; print(site.getsitepackages()[0])")
-
-  LD_LIBRARY_PATH=$PWD/release:$LD_LIBRARY_PATH PYTHONPATH=$pythonpath ctest \
-    --exclude-regex "plasma-serialization_tests" \
-    --label-regex unittest \
-    --output-on-failure \
-    --parallel $NPROC \
-    --timeout 300
+  if [ ${TEST_CPP} -gt 0 ]; then
+    LD_LIBRARY_PATH=$PWD/release:$LD_LIBRARY_PATH ctest \
+      --label-regex unittest \
+      --output-on-failure \
+      --parallel $NPROC \
+      --timeout 300
+  fi
 
   popd
 }
@@ -657,8 +756,8 @@ test_python() {
   show_header "Build and test Python libraries"
 
   # Build and test Python
-  maybe_setup_virtualenv cython numpy setuptools_scm setuptools || exit 1
-  maybe_setup_conda --file ci/conda_env_python.txt || exit 1
+  maybe_setup_virtualenv
+  maybe_setup_conda --file ci/conda_env_python.txt
 
   if [ "${USE_CONDA}" -gt 0 ]; then
     CMAKE_PREFIX_PATH="${CONDA_BACKUP_CMAKE_PREFIX_PATH}:${CMAKE_PREFIX_PATH}"
@@ -682,9 +781,6 @@ test_python() {
   if [ "${ARROW_GCS}" = "ON" ]; then
     export PYARROW_WITH_GCS=1
   fi
-  if [ "${ARROW_PLASMA}" = "ON" ]; then
-    export PYARROW_WITH_PLASMA=1
-  fi
   if [ "${ARROW_S3}" = "ON" ]; then
     export PYARROW_WITH_S3=1
   fi
@@ -692,7 +788,7 @@ test_python() {
   pushd python
 
   # Build pyarrow
-  python setup.py build_ext --inplace
+  python -m pip install -e .
 
   # Check mandatory and optional imports
   python -c "
@@ -717,9 +813,6 @@ import pyarrow.parquet
   if [ "${ARROW_GCS}" == "ON" ]; then
     python -c "import pyarrow._gcsfs"
   fi
-  if [ "${ARROW_PLASMA}" == "ON" ]; then
-    python -c "import pyarrow.plasma"
-  fi
   if [ "${ARROW_S3}" == "ON" ]; then
     python -c "import pyarrow._s3fs"
   fi
@@ -738,8 +831,8 @@ test_glib() {
   show_header "Build and test C GLib libraries"
 
   # Build and test C GLib
-  maybe_setup_conda glib gobject-introspection meson ninja ruby || exit 1
-  maybe_setup_virtualenv meson || exit 1
+  maybe_setup_conda glib gobject-introspection meson ninja ruby
+  maybe_setup_virtualenv meson
 
   # Install bundler if doesn't exist
   if ! bundle --version; then
@@ -773,8 +866,8 @@ test_ruby() {
   show_header "Build and test Ruby libraries"
 
   # required dependencies are installed by test_glib
-  maybe_setup_conda || exit 1
-  maybe_setup_virtualenv || exit 1
+  maybe_setup_conda
+  maybe_setup_virtualenv
 
   which ruby
   which bundle
@@ -794,9 +887,6 @@ test_ruby() {
   if [ "${ARROW_GANDIVA}" = "ON" ]; then
     modules="${modules} red-gandiva"
   fi
-  if [ "${ARROW_PLASMA}" = "ON" ]; then
-    modules="${modules} red-plasma"
-  fi
 
   for module in ${modules}; do
     pushd ${module}
@@ -826,8 +916,12 @@ test_csharp() {
     mv ../.git dummy.git
   fi
 
-  sourcelink test artifacts/Apache.Arrow/Release/netstandard1.3/Apache.Arrow.pdb
-  sourcelink test artifacts/Apache.Arrow/Release/netcoreapp3.1/Apache.Arrow.pdb
+  if [ "${SOURCE_KIND}" = "local" ]; then
+    echo "Skipping sourcelink verification on local build"
+  else
+    dotnet tool run sourcelink test artifacts/Apache.Arrow/Release/netstandard2.0/Apache.Arrow.pdb
+    dotnet tool run sourcelink test artifacts/Apache.Arrow/Release/net6.0/Apache.Arrow.pdb
+  fi
 
   popd
 }
@@ -835,11 +929,12 @@ test_csharp() {
 test_js() {
   show_header "Build and test JavaScript libraries"
 
-  maybe_setup_nodejs || exit 1
-  maybe_setup_conda nodejs=16 || exit 1
+  maybe_setup_nodejs
+  maybe_setup_conda nodejs=18
 
   if ! command -v yarn &> /dev/null; then
-    npm install -g yarn
+    npm install yarn
+    PATH=$PWD/node_modules/yarn/bin:$PATH
   fi
 
   pushd js
@@ -847,20 +942,41 @@ test_js() {
   yarn clean:all
   yarn lint
   yarn build
-  yarn test
-  yarn test:bundle
+  if [ ${TEST_JS} -gt 0 ]; then
+    yarn test
+    yarn test:bundle
+  fi
   popd
 }
 
 test_go() {
   show_header "Build and test Go libraries"
 
-  maybe_setup_go || exit 1
-  maybe_setup_conda compilers go=1.17 || exit 1
+  maybe_setup_go
+  maybe_setup_conda compilers go=1.21
 
-  pushd go/arrow
+  pushd go
   go get -v ./...
-  go test ./...
+  if [ ${TEST_GO} -gt 0 ]; then
+    go test ./...
+  fi
+  go install -buildvcs=false ./...
+  if [ ${TEST_INTEGRATION_GO} -gt 0 ]; then
+    pushd arrow/internal/cdata_integration
+    case "$(uname)" in
+      Linux)
+        go_lib="arrow_go_integration.so"
+        ;;
+      Darwin)
+        go_lib="arrow_go_integration.dylib"
+        ;;
+      MINGW*)
+        go_lib="arrow_go_integration.dll"
+        ;;
+    esac
+    CGO_ENABLED=1 go build -buildvcs=false -tags cdata_integration,assert -buildmode=c-shared -o ${go_lib} .
+    popd
+  fi
   go clean -modcache
   popd
 }
@@ -869,10 +985,11 @@ test_go() {
 test_integration() {
   show_header "Build and execute integration tests"
 
-  maybe_setup_conda || exit 1
-  maybe_setup_virtualenv || exit 1
+  maybe_setup_conda
+  maybe_setup_virtualenv
 
-  pip install -e dev/archery
+  pip install -e dev/archery[integration]
+  pip install -e dev/archery[integration-java]
 
   JAVA_DIR=$ARROW_SOURCE_DIR/java
   CPP_BUILD_DIR=$ARROW_TMPDIR/cpp-build
@@ -888,6 +1005,7 @@ test_integration() {
 
   # Flight integration test executable have runtime dependency on release/libgtest.so
   LD_LIBRARY_PATH=$ARROW_CPP_EXE_PATH:$LD_LIBRARY_PATH archery integration \
+    --run-ipc --run-flight --run-c-data \
     --with-cpp=${TEST_INTEGRATION_CPP} \
     --with-java=${TEST_INTEGRATION_JAVA} \
     --with-js=${TEST_INTEGRATION_JS} \
@@ -927,12 +1045,26 @@ ensure_source_directory() {
     fi
   fi
 
-  # Ensure that the testing repositories are cloned
-  if [ ! -d "${ARROW_SOURCE_DIR}/testing/data" ]; then
-    git clone https://github.com/apache/arrow-testing.git ${ARROW_SOURCE_DIR}/testing
+  # Ensure that the testing repositories are prepared
+  if [ ! -d ${ARROW_SOURCE_DIR}/testing/data ]; then
+    if [ -d ${SOURCE_DIR}/../../testing/data ]; then
+      cp -a ${SOURCE_DIR}/../../testing ${ARROW_SOURCE_DIR}/
+    else
+      git clone \
+        https://github.com/apache/arrow-testing.git \
+        ${ARROW_SOURCE_DIR}/testing
+    fi
   fi
-  if [ ! -d "${ARROW_SOURCE_DIR}/cpp/submodules/parquet-testing/data" ]; then
-    git clone https://github.com/apache/parquet-testing.git ${ARROW_SOURCE_DIR}/cpp/submodules/parquet-testing
+  if [ ! -d ${ARROW_SOURCE_DIR}/cpp/submodules/parquet-testing/data ]; then
+    if [ -d ${SOURCE_DIR}/../../cpp/submodules/parquet-testing/data ]; then
+      cp -a \
+         ${SOURCE_DIR}/../../cpp/submodules/parquet-testing \
+         ${ARROW_SOURCE_DIR}/cpp/submodules/
+    else
+      git clone \
+        https://github.com/apache/parquet-testing.git \
+        ${ARROW_SOURCE_DIR}/cpp/submodules/parquet-testing
+    fi
   fi
 
   export ARROW_TEST_DATA=$ARROW_SOURCE_DIR/testing/data
@@ -956,16 +1088,16 @@ test_source_distribution() {
 
   pushd $ARROW_SOURCE_DIR
 
-  if [ ${TEST_GO} -gt 0 ]; then
+  if [ ${BUILD_GO} -gt 0 ]; then
     test_go
   fi
   if [ ${TEST_CSHARP} -gt 0 ]; then
     test_csharp
   fi
-  if [ ${TEST_JS} -gt 0 ]; then
+  if [ ${BUILD_JS} -gt 0 ]; then
     test_js
   fi
-  if [ ${TEST_CPP} -gt 0 ]; then
+  if [ ${BUILD_CPP} -gt 0 ]; then
     test_and_install_cpp
   fi
   if [ ${TEST_PYTHON} -gt 0 ]; then
@@ -977,7 +1109,7 @@ test_source_distribution() {
   if [ ${TEST_RUBY} -gt 0 ]; then
     test_ruby
   fi
-  if [ ${TEST_JAVA} -gt 0 ]; then
+  if [ ${BUILD_JAVA} -gt 0 ]; then
     test_package_java
   fi
   if [ ${TEST_INTEGRATION} -gt 0 ]; then
@@ -1014,15 +1146,17 @@ test_linux_wheels() {
     local arch="x86_64"
   fi
 
-  local python_versions="${TEST_PYTHON_VERSIONS:-3.7m 3.8 3.9 3.10}"
-  local platform_tags="manylinux_2_17_${arch}.manylinux2014_${arch}"
+  local python_versions="${TEST_PYTHON_VERSIONS:-3.8 3.9 3.10 3.11 3.12}"
+  local platform_tags="${TEST_WHEEL_PLATFORM_TAGS:-manylinux_2_17_${arch}.manylinux2014_${arch} manylinux_2_28_${arch}}"
 
   for python in ${python_versions}; do
     local pyver=${python/m}
     for platform in ${platform_tags}; do
       show_header "Testing Python ${pyver} wheel for platform ${platform}"
-      CONDA_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_conda || exit 1
-      VENV_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_virtualenv || continue
+      CONDA_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_conda
+      if ! VENV_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_virtualenv; then
+        continue
+      fi
       pip install pyarrow-${TEST_PYARROW_VERSION:-${VERSION}}-cp${pyver/.}-cp${python/.}-${platform}.whl
       INSTALL_PYARROW=OFF ARROW_GCS=${check_gcs} ${ARROW_DIR}/ci/scripts/python_wheel_unix_test.sh ${ARROW_SOURCE_DIR}
     done
@@ -1036,12 +1170,12 @@ test_macos_wheels() {
 
   # apple silicon processor
   if [ "$(uname -m)" = "arm64" ]; then
-    local python_versions="3.8 3.9 3.10"
+    local python_versions="3.8 3.9 3.10 3.11 3.12"
     local platform_tags="macosx_11_0_arm64"
     local check_flight=OFF
   else
-    local python_versions="3.7m 3.8 3.9 3.10"
-    local platform_tags="macosx_10_14_x86_64"
+    local python_versions="3.8 3.9 3.10 3.11 3.12"
+    local platform_tags="macosx_10_15_x86_64"
   fi
 
   # verify arch-native wheels inside an arch-native conda environment
@@ -1054,38 +1188,21 @@ test_macos_wheels() {
         check_s3=OFF
       fi
 
-      CONDA_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_conda || exit 1
-      VENV_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_virtualenv || continue
+      CONDA_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_conda
+      if ! VENV_ENV=wheel-${pyver}-${platform} PYTHON_VERSION=${pyver} maybe_setup_virtualenv; then
+        continue
+      fi
 
       pip install pyarrow-${VERSION}-cp${pyver/.}-cp${python/.}-${platform}.whl
       INSTALL_PYARROW=OFF ARROW_FLIGHT=${check_flight} ARROW_GCS=${check_gcs} ARROW_S3=${check_s3} \
         ${ARROW_DIR}/ci/scripts/python_wheel_unix_test.sh ${ARROW_SOURCE_DIR}
     done
   done
-
-  # verify arm64 and universal2 wheels using an universal2 python binary
-  # the interpreter should be installed from python.org:
-  #   https://www.python.org/ftp/python/3.9.6/python-3.9.6-macosx10.9.pkg
-  if [ "$(uname -m)" = "arm64" ]; then
-    for pyver in "3.9 3.10"; do
-      local python="/Library/Frameworks/Python.framework/Versions/${pyver}/bin/python${pyver}"
-
-      # create and activate a virtualenv for testing as arm64
-      for arch in "arm64" "x86_64"; do
-        VENV_ENV=wheel-${pyver}-universal2-${arch} PYTHON=${python} maybe_setup_virtualenv || continue
-        # install pyarrow's universal2 wheel
-        pip install pyarrow-${VERSION}-cp${pyver/.}-cp${pyver/.}-macosx_11_0_universal2.whl
-        # check the imports and execute the unittests
-        INSTALL_PYARROW=OFF ARROW_FLIGHT=${check_flight} \
-          arch -${arch} ${ARROW_DIR}/ci/scripts/python_wheel_unix_test.sh ${ARROW_SOURCE_DIR}
-      done
-    done
-  fi
 }
 
 test_wheels() {
   show_header "Downloading Python wheels"
-  maybe_setup_conda python || exit 1
+  maybe_setup_conda python
 
   local wheels_dir=
   if [ "${SOURCE_KIND}" = "local" ]; then
@@ -1124,9 +1241,11 @@ test_wheels() {
 
 test_jars() {
   show_header "Testing Java JNI jars"
-  maybe_setup_conda maven python || exit 1
 
-  local download_dir=jars
+  maybe_setup_maven
+  maybe_setup_conda maven python
+
+  local download_dir=${ARROW_TMPDIR}/jars
   mkdir -p ${download_dir}
 
   ${PYTHON:-python3} $SOURCE_DIR/download_rc_binaries.py $VERSION $RC_NUMBER \
@@ -1177,15 +1296,15 @@ test_jars() {
 : ${TEST_INTEGRATION_JS:=${TEST_INTEGRATION}}
 : ${TEST_INTEGRATION_GO:=${TEST_INTEGRATION}}
 
-# Automatically test if its activated by a dependent
+# Automatically build/test if its activated by a dependent
 TEST_GLIB=$((${TEST_GLIB} + ${TEST_RUBY}))
-TEST_CPP=$((${TEST_CPP} + ${TEST_GLIB} + ${TEST_PYTHON} + ${TEST_INTEGRATION_CPP}))
-TEST_JAVA=$((${TEST_JAVA} + ${TEST_INTEGRATION_JAVA}))
-TEST_JS=$((${TEST_JS} + ${TEST_INTEGRATION_JS}))
-TEST_GO=$((${TEST_GO} + ${TEST_INTEGRATION_GO}))
+BUILD_CPP=$((${TEST_CPP} + ${TEST_GLIB} + ${TEST_PYTHON} + ${TEST_INTEGRATION_CPP}))
+BUILD_JAVA=$((${TEST_JAVA} + ${TEST_INTEGRATION_JAVA}))
+BUILD_JS=$((${TEST_JS} + ${TEST_INTEGRATION_JS}))
+BUILD_GO=$((${TEST_GO} + ${TEST_INTEGRATION_GO}))
 TEST_INTEGRATION=$((${TEST_INTEGRATION} + ${TEST_INTEGRATION_CPP} + ${TEST_INTEGRATION_JAVA} + ${TEST_INTEGRATION_JS} + ${TEST_INTEGRATION_GO}))
 
-# Execute tests in a conda enviroment
+# Execute tests in a conda environment
 : ${USE_CONDA:=0}
 
 # Build options for the C++ library
@@ -1197,7 +1316,6 @@ fi
 : ${ARROW_FLIGHT:=ON}
 : ${ARROW_GANDIVA:=ON}
 : ${ARROW_GCS:=OFF}
-: ${ARROW_PLASMA:=ON}
 : ${ARROW_S3:=OFF}
 
 TEST_SUCCESS=no
@@ -1209,5 +1327,5 @@ test_binary_distribution
 
 TEST_SUCCESS=yes
 
-echo 'Release candidate looks good!'
+echo "Release candidate ${VERSION}-RC${RC_NUMBER} looks good!"
 exit 0
