@@ -14,17 +14,21 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+//go:build go1.18
+
 package kernels
 
 import (
 	"fmt"
 	"unsafe"
 
-	"github.com/apache/arrow/go/v10/arrow"
-	"github.com/apache/arrow/go/v10/arrow/bitutil"
-	"github.com/apache/arrow/go/v10/arrow/compute/internal/exec"
-	"github.com/apache/arrow/go/v10/arrow/memory"
-	"github.com/apache/arrow/go/v10/internal/bitutils"
+	"github.com/apache/arrow/go/v17/arrow"
+	"github.com/apache/arrow/go/v17/arrow/bitutil"
+	"github.com/apache/arrow/go/v17/arrow/compute/exec"
+	"github.com/apache/arrow/go/v17/arrow/internal/debug"
+	"github.com/apache/arrow/go/v17/arrow/memory"
+	"github.com/apache/arrow/go/v17/arrow/scalar"
+	"github.com/apache/arrow/go/v17/internal/bitutils"
 	"golang.org/x/exp/constraints"
 )
 
@@ -33,9 +37,9 @@ import (
 // which will receive a slice containing the raw input data along with
 // a slice to populate for the output data.
 //
-// Note that bool is not included in exec.FixedWidthTypes since it is
+// Note that bool is not included in arrow.FixedWidthType since it is
 // represented as a bitmap, not as a slice of bool.
-func ScalarUnary[OutT, Arg0T exec.FixedWidthTypes](op func(*exec.KernelCtx, []Arg0T, []OutT) error) exec.ArrayKernelExec {
+func ScalarUnary[OutT, Arg0T arrow.FixedWidthType](op func(*exec.KernelCtx, []Arg0T, []OutT) error) exec.ArrayKernelExec {
 	return func(ctx *exec.KernelCtx, in *exec.ExecSpan, out *exec.ExecResult) error {
 		arg0 := in.Values[0].Array
 		inData := exec.GetSpanValues[Arg0T](&arg0, 1)
@@ -47,7 +51,7 @@ func ScalarUnary[OutT, Arg0T exec.FixedWidthTypes](op func(*exec.KernelCtx, []Ar
 // ScalarUnaryNotNull is for generating a kernel to operate only on the
 // non-null values in the input array. The zerovalue of the output type
 // is used for any null input values.
-func ScalarUnaryNotNull[OutT, Arg0T exec.FixedWidthTypes](op func(*exec.KernelCtx, Arg0T, *error) OutT) exec.ArrayKernelExec {
+func ScalarUnaryNotNull[OutT, Arg0T arrow.FixedWidthType](op func(*exec.KernelCtx, Arg0T, *error) OutT) exec.ArrayKernelExec {
 	return func(ctx *exec.KernelCtx, in *exec.ExecSpan, out *exec.ExecResult) error {
 		var (
 			arg0     = &in.Values[0].Array
@@ -74,7 +78,7 @@ func ScalarUnaryNotNull[OutT, Arg0T exec.FixedWidthTypes](op func(*exec.KernelCt
 // ScalarUnaryBoolOutput is like ScalarUnary only it is for cases of boolean
 // output. The function should take in a slice of the input type and a slice
 // of bytes to fill with the output boolean bitmap.
-func ScalarUnaryBoolOutput[Arg0T exec.FixedWidthTypes](op func(*exec.KernelCtx, []Arg0T, []byte) error) exec.ArrayKernelExec {
+func ScalarUnaryBoolOutput[Arg0T arrow.FixedWidthType](op func(*exec.KernelCtx, []Arg0T, []byte) error) exec.ArrayKernelExec {
 	return func(ctx *exec.KernelCtx, in *exec.ExecSpan, out *exec.ExecResult) error {
 		arg0 := in.Values[0].Array
 		inData := exec.GetSpanValues[Arg0T](&arg0, 1)
@@ -123,7 +127,7 @@ func ScalarUnaryNotNullBinaryArgBoolOut[OffsetT int32 | int64](defVal bool, op f
 // It implements the handling to iterate the offsets and values calling
 // the provided function on each byte slice. The zero value of the OutT
 // will be used as the output for elements of the input that are null.
-func ScalarUnaryNotNullBinaryArg[OutT exec.FixedWidthTypes, OffsetT int32 | int64](op func(*exec.KernelCtx, []byte, *error) OutT) exec.ArrayKernelExec {
+func ScalarUnaryNotNullBinaryArg[OutT arrow.FixedWidthType, OffsetT int32 | int64](op func(*exec.KernelCtx, []byte, *error) OutT) exec.ArrayKernelExec {
 	return func(ctx *exec.KernelCtx, in *exec.ExecSpan, out *exec.ExecResult) error {
 		var (
 			arg0        = &in.Values[0].Array
@@ -152,13 +156,285 @@ func ScalarUnaryNotNullBinaryArg[OutT exec.FixedWidthTypes, OffsetT int32 | int6
 // ScalarUnaryBoolArg is like ScalarUnary except it specifically expects a
 // function that takes a byte slice since booleans arrays are represented
 // as a bitmap.
-func ScalarUnaryBoolArg[OutT exec.FixedWidthTypes](op func(*exec.KernelCtx, []byte, []OutT) error) exec.ArrayKernelExec {
+func ScalarUnaryBoolArg[OutT arrow.FixedWidthType](op func(*exec.KernelCtx, []byte, []OutT) error) exec.ArrayKernelExec {
 	return func(ctx *exec.KernelCtx, input *exec.ExecSpan, out *exec.ExecResult) error {
 		outData := exec.GetSpanValues[OutT](out, 1)
 		return op(ctx, input.Values[0].Array.Buffers[1].Buf, outData)
 	}
 }
 
+func UnboxScalar[T arrow.FixedWidthType](val scalar.PrimitiveScalar) T {
+	return *(*T)(unsafe.Pointer(&val.Data()[0]))
+}
+
+func UnboxBinaryScalar(val scalar.BinaryScalar) []byte {
+	if !val.IsValid() {
+		return nil
+	}
+	return val.Data()
+}
+
+type arrArrFn[OutT, Arg0T, Arg1T arrow.FixedWidthType] func(*exec.KernelCtx, []Arg0T, []Arg1T, []OutT) error
+type arrScalarFn[OutT, Arg0T, Arg1T arrow.FixedWidthType] func(*exec.KernelCtx, []Arg0T, Arg1T, []OutT) error
+type scalarArrFn[OutT, Arg0T, Arg1T arrow.FixedWidthType] func(*exec.KernelCtx, Arg0T, []Arg1T, []OutT) error
+
+type binaryOps[OutT, Arg0T, Arg1T arrow.FixedWidthType] struct {
+	arrArr    arrArrFn[OutT, Arg0T, Arg1T]
+	arrScalar arrScalarFn[OutT, Arg0T, Arg1T]
+	scalarArr scalarArrFn[OutT, Arg0T, Arg1T]
+}
+
+type binaryBoolOps struct {
+	arrArr    func(ctx *exec.KernelCtx, lhs, rhs, out bitutil.Bitmap) error
+	arrScalar func(ctx *exec.KernelCtx, lhs bitutil.Bitmap, rhs bool, out bitutil.Bitmap) error
+	scalarArr func(ctx *exec.KernelCtx, lhs bool, rhs, out bitutil.Bitmap) error
+}
+
+func ScalarBinary[OutT, Arg0T, Arg1T arrow.FixedWidthType](ops binaryOps[OutT, Arg0T, Arg1T]) exec.ArrayKernelExec {
+	arrayArray := func(ctx *exec.KernelCtx, arg0, arg1 *exec.ArraySpan, out *exec.ExecResult) error {
+		var (
+			a0      = exec.GetSpanValues[Arg0T](arg0, 1)
+			a1      = exec.GetSpanValues[Arg1T](arg1, 1)
+			outData = exec.GetSpanValues[OutT](out, 1)
+		)
+		return ops.arrArr(ctx, a0, a1, outData)
+	}
+
+	arrayScalar := func(ctx *exec.KernelCtx, arg0 *exec.ArraySpan, arg1 scalar.Scalar, out *exec.ExecResult) error {
+		var (
+			a0      = exec.GetSpanValues[Arg0T](arg0, 1)
+			a1      = UnboxScalar[Arg1T](arg1.(scalar.PrimitiveScalar))
+			outData = exec.GetSpanValues[OutT](out, 1)
+		)
+		return ops.arrScalar(ctx, a0, a1, outData)
+	}
+
+	scalarArray := func(ctx *exec.KernelCtx, arg0 scalar.Scalar, arg1 *exec.ArraySpan, out *exec.ExecResult) error {
+		var (
+			a0      = UnboxScalar[Arg0T](arg0.(scalar.PrimitiveScalar))
+			a1      = exec.GetSpanValues[Arg1T](arg1, 1)
+			outData = exec.GetSpanValues[OutT](out, 1)
+		)
+		return ops.scalarArr(ctx, a0, a1, outData)
+	}
+
+	return func(ctx *exec.KernelCtx, batch *exec.ExecSpan, out *exec.ExecResult) error {
+		if batch.Values[0].IsArray() {
+			if batch.Values[1].IsArray() {
+				return arrayArray(ctx, &batch.Values[0].Array, &batch.Values[1].Array, out)
+			}
+			return arrayScalar(ctx, &batch.Values[0].Array, batch.Values[1].Scalar, out)
+		}
+
+		if batch.Values[1].IsArray() {
+			return scalarArray(ctx, batch.Values[0].Scalar, &batch.Values[1].Array, out)
+		}
+
+		debug.Assert(false, "should be unreachable")
+		return fmt.Errorf("%w: scalar binary with two scalars?", arrow.ErrInvalid)
+	}
+}
+
+func ScalarBinaryBools(ops *binaryBoolOps) exec.ArrayKernelExec {
+	arrayArray := func(ctx *exec.KernelCtx, arg0, arg1 *exec.ArraySpan, out *exec.ExecResult) error {
+		var (
+			a0Bm  = bitutil.Bitmap{Data: arg0.Buffers[1].Buf, Offset: arg0.Offset, Len: arg0.Len}
+			a1Bm  = bitutil.Bitmap{Data: arg1.Buffers[1].Buf, Offset: arg1.Offset, Len: arg1.Len}
+			outBm = bitutil.Bitmap{Data: out.Buffers[1].Buf, Offset: out.Offset, Len: out.Len}
+		)
+
+		return ops.arrArr(ctx, a0Bm, a1Bm, outBm)
+	}
+
+	arrayScalar := func(ctx *exec.KernelCtx, arg0 *exec.ArraySpan, arg1 scalar.Scalar, out *exec.ExecResult) error {
+		var (
+			a0Bm  = bitutil.Bitmap{Data: arg0.Buffers[1].Buf, Offset: arg0.Offset, Len: arg0.Len}
+			a1    = arg1.(*scalar.Boolean).Value
+			outBm = bitutil.Bitmap{Data: out.Buffers[1].Buf, Offset: out.Offset, Len: out.Len}
+		)
+		return ops.arrScalar(ctx, a0Bm, a1, outBm)
+	}
+
+	scalarArray := func(ctx *exec.KernelCtx, arg0 scalar.Scalar, arg1 *exec.ArraySpan, out *exec.ExecResult) error {
+		var (
+			a0    = arg0.(*scalar.Boolean).Value
+			a1Bm  = bitutil.Bitmap{Data: arg1.Buffers[1].Buf, Offset: arg1.Offset, Len: arg1.Len}
+			outBm = bitutil.Bitmap{Data: out.Buffers[1].Buf, Offset: out.Offset, Len: out.Len}
+		)
+		return ops.scalarArr(ctx, a0, a1Bm, outBm)
+	}
+
+	return func(ctx *exec.KernelCtx, batch *exec.ExecSpan, out *exec.ExecResult) error {
+		if batch.Values[0].IsArray() {
+			if batch.Values[1].IsArray() {
+				return arrayArray(ctx, &batch.Values[0].Array, &batch.Values[1].Array, out)
+			}
+			return arrayScalar(ctx, &batch.Values[0].Array, batch.Values[1].Scalar, out)
+		}
+
+		if batch.Values[1].IsArray() {
+			return scalarArray(ctx, batch.Values[0].Scalar, &batch.Values[1].Array, out)
+		}
+
+		debug.Assert(false, "should be unreachable")
+		return fmt.Errorf("%w: scalar binary with two scalars?", arrow.ErrInvalid)
+	}
+}
+
+func ScalarBinaryNotNull[OutT, Arg0T, Arg1T arrow.FixedWidthType](op func(*exec.KernelCtx, Arg0T, Arg1T, *error) OutT) exec.ArrayKernelExec {
+	arrayArray := func(ctx *exec.KernelCtx, arg0, arg1 *exec.ArraySpan, out *exec.ExecResult) (err error) {
+		// fast path if one side is entirely null
+		if arg0.UpdateNullCount() == arg0.Len || arg1.UpdateNullCount() == arg1.Len {
+			return nil
+		}
+
+		var (
+			a0      = exec.GetSpanValues[Arg0T](arg0, 1)
+			a1      = exec.GetSpanValues[Arg1T](arg1, 1)
+			outData = exec.GetSpanValues[OutT](out, 1)
+			outPos  int64
+			def     OutT
+		)
+		bitutils.VisitTwoBitBlocks(arg0.Buffers[0].Buf, arg1.Buffers[0].Buf, arg0.Offset, arg1.Offset, out.Len,
+			func(pos int64) {
+				outData[outPos] = op(ctx, a0[pos], a1[pos], &err)
+				outPos++
+			}, func() {
+				outData[outPos] = def
+				outPos++
+			})
+		return
+	}
+
+	arrayScalar := func(ctx *exec.KernelCtx, arg0 *exec.ArraySpan, arg1 scalar.Scalar, out *exec.ExecResult) (err error) {
+		// fast path if one side is entirely null
+		if arg0.UpdateNullCount() == arg0.Len || !arg1.IsValid() {
+			return nil
+		}
+
+		var (
+			a0      = exec.GetSpanValues[Arg0T](arg0, 1)
+			outData = exec.GetSpanValues[OutT](out, 1)
+			outPos  int64
+			def     OutT
+		)
+		if !arg1.IsValid() {
+			return nil
+		}
+
+		a1 := UnboxScalar[Arg1T](arg1.(scalar.PrimitiveScalar))
+		bitutils.VisitBitBlocks(arg0.Buffers[0].Buf, arg0.Offset, arg0.Len,
+			func(pos int64) {
+				outData[outPos] = op(ctx, a0[pos], a1, &err)
+				outPos++
+			}, func() {
+				outData[outPos] = def
+				outPos++
+			})
+		return
+	}
+
+	scalarArray := func(ctx *exec.KernelCtx, arg0 scalar.Scalar, arg1 *exec.ArraySpan, out *exec.ExecResult) (err error) {
+		// fast path if one side is entirely null
+		if arg1.UpdateNullCount() == arg1.Len || !arg0.IsValid() {
+			return nil
+		}
+
+		var (
+			a1      = exec.GetSpanValues[Arg1T](arg1, 1)
+			outData = exec.GetSpanValues[OutT](out, 1)
+			outPos  int64
+			def     OutT
+		)
+		if !arg0.IsValid() {
+			return nil
+		}
+
+		a0 := UnboxScalar[Arg0T](arg0.(scalar.PrimitiveScalar))
+		bitutils.VisitBitBlocks(arg1.Buffers[0].Buf, arg1.Offset, arg1.Len,
+			func(pos int64) {
+				outData[outPos] = op(ctx, a0, a1[pos], &err)
+				outPos++
+			}, func() {
+				outData[outPos] = def
+				outPos++
+			})
+		return
+	}
+
+	return func(ctx *exec.KernelCtx, batch *exec.ExecSpan, out *exec.ExecResult) error {
+		if batch.Values[0].IsArray() {
+			if batch.Values[1].IsArray() {
+				return arrayArray(ctx, &batch.Values[0].Array, &batch.Values[1].Array, out)
+			}
+			return arrayScalar(ctx, &batch.Values[0].Array, batch.Values[1].Scalar, out)
+		}
+
+		if batch.Values[1].IsArray() {
+			return scalarArray(ctx, batch.Values[0].Scalar, &batch.Values[1].Array, out)
+		}
+
+		debug.Assert(false, "should be unreachable")
+		return fmt.Errorf("%w: scalar binary with two scalars?", arrow.ErrInvalid)
+	}
+}
+
+type binaryBinOp[T arrow.FixedWidthType | bool] func(ctx *exec.KernelCtx, arg0, arg1 []byte) T
+
+func ScalarBinaryBinaryArgsBoolOut(itrFn func(*exec.ArraySpan) exec.ArrayIter[[]byte], op binaryBinOp[bool]) exec.ArrayKernelExec {
+	arrArr := func(ctx *exec.KernelCtx, arg0, arg1 *exec.ArraySpan, out *exec.ExecResult) error {
+		var (
+			arg0It = itrFn(arg0)
+			arg1It = itrFn(arg1)
+		)
+
+		bitutils.GenerateBitsUnrolled(out.Buffers[1].Buf, out.Offset, out.Len, func() bool {
+			return op(ctx, arg0It.Next(), arg1It.Next())
+		})
+		return nil
+	}
+
+	arrScalar := func(ctx *exec.KernelCtx, arg0 *exec.ArraySpan, arg1 scalar.Scalar, out *exec.ExecResult) error {
+		var (
+			arg0It = itrFn(arg0)
+			a1     = UnboxBinaryScalar(arg1.(scalar.BinaryScalar))
+		)
+
+		bitutils.GenerateBitsUnrolled(out.Buffers[1].Buf, out.Offset, out.Len, func() bool {
+			return op(ctx, arg0It.Next(), a1)
+		})
+		return nil
+	}
+
+	scalarArr := func(ctx *exec.KernelCtx, arg0 scalar.Scalar, arg1 *exec.ArraySpan, out *exec.ExecResult) error {
+		var (
+			arg1It = itrFn(arg1)
+			a0     = UnboxBinaryScalar(arg0.(scalar.BinaryScalar))
+		)
+
+		bitutils.GenerateBitsUnrolled(out.Buffers[1].Buf, out.Offset, out.Len, func() bool {
+			return op(ctx, a0, arg1It.Next())
+		})
+		return nil
+	}
+
+	return func(ctx *exec.KernelCtx, batch *exec.ExecSpan, out *exec.ExecResult) error {
+		if batch.Values[0].IsArray() {
+			if batch.Values[1].IsArray() {
+				return arrArr(ctx, &batch.Values[0].Array, &batch.Values[1].Array, out)
+			}
+			return arrScalar(ctx, &batch.Values[0].Array, batch.Values[1].Scalar, out)
+		}
+
+		if batch.Values[1].IsArray() {
+			return scalarArr(ctx, batch.Values[0].Scalar, &batch.Values[1].Array, out)
+		}
+
+		debug.Assert(false, "should be unreachable")
+		return fmt.Errorf("%w: scalar binary with two scalars?", arrow.ErrInvalid)
+	}
+}
+
 // SizeOf determines the size in number of bytes for an integer
 // based on the generic value in a way that the compiler should
 // be able to easily evaluate and create as a constant.
@@ -301,7 +577,7 @@ func intsCanFit(data *exec.ArraySpan, target arrow.Type) error {
 	}
 }
 
-func intsInRange[T exec.IntTypes | exec.UintTypes](data *exec.ArraySpan, lowerBound, upperBound T) error {
+func intsInRange[T arrow.IntType | arrow.UintType](data *exec.ArraySpan, lowerBound, upperBound T) error {
 	if MinOf[T]() >= lowerBound && MaxOf[T]() <= upperBound {
 		return nil
 	}
@@ -377,7 +653,7 @@ func intsInRange[T exec.IntTypes | exec.UintTypes](data *exec.ArraySpan, lowerBo
 }
 
 type numeric interface {
-	exec.IntTypes | exec.UintTypes | constraints.Float
+	arrow.IntType | arrow.UintType | constraints.Float
 }
 
 func memCpySpan[T numeric](in, out *exec.ArraySpan) {
@@ -422,7 +698,7 @@ func castNumberToNumberUnsafe(in, out *exec.ArraySpan) {
 	castNumericUnsafe(in.Type.ID(), out.Type.ID(), in.Buffers[1].Buf[inputOffset:], out.Buffers[1].Buf[outputOffset:], int(in.Len))
 }
 
-func maxDecimalDigitsForInt(id arrow.Type) (int32, error) {
+func MaxDecimalDigitsForInt(id arrow.Type) (int32, error) {
 	switch id {
 	case arrow.INT8, arrow.UINT8:
 		return 3, nil
@@ -445,11 +721,67 @@ func ResolveOutputFromOptions(ctx *exec.KernelCtx, _ []arrow.DataType) (arrow.Da
 
 var OutputTargetType = exec.NewComputedOutputType(ResolveOutputFromOptions)
 
-func resolveToFirstType(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
+var OutputFirstType = exec.NewComputedOutputType(func(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
 	return args[0], nil
+})
+
+var OutputLastType = exec.NewComputedOutputType(func(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
+	return args[len(args)-1], nil
+})
+
+func resolveDecimalBinaryOpOutput(types []arrow.DataType, resolver func(prec1, scale1, prec2, scale2 int32) (prec, scale int32)) (arrow.DataType, error) {
+	leftType, rightType := types[0].(arrow.DecimalType), types[1].(arrow.DecimalType)
+	debug.Assert(leftType.ID() == rightType.ID(), "decimal binary ops should have casted to the same type")
+
+	prec, scale := resolver(leftType.GetPrecision(), leftType.GetScale(),
+		rightType.GetPrecision(), rightType.GetScale())
+
+	return arrow.NewDecimalType(leftType.ID(), prec, scale)
 }
 
-var OutputFirstType = exec.NewComputedOutputType(resolveToFirstType)
+func resolveDecimalAddOrSubtractType(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
+	return resolveDecimalBinaryOpOutput(args,
+		func(prec1, scale1, prec2, scale2 int32) (prec int32, scale int32) {
+			debug.Assert(scale1 == scale2, "decimal operations should use the same scale")
+			scale = scale1
+			prec = exec.Max(prec1-scale1, prec2-scale2) + scale + 1
+			return
+		})
+}
+
+func resolveDecimalMultiplyOutput(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
+	return resolveDecimalBinaryOpOutput(args,
+		func(prec1, scale1, prec2, scale2 int32) (prec int32, scale int32) {
+			scale = scale1 + scale2
+			prec = prec1 + prec2 + 1
+			return
+		})
+}
+
+func resolveDecimalDivideOutput(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
+	return resolveDecimalBinaryOpOutput(args,
+		func(prec1, scale1, prec2, scale2 int32) (prec int32, scale int32) {
+			debug.Assert(scale1 >= scale2, "when dividing decimal values numerator scale should be greater/equal to denom scale")
+			scale = scale1 - scale2
+			prec = prec1
+			return
+		})
+}
+
+func resolveTemporalOutput(_ *exec.KernelCtx, args []arrow.DataType) (arrow.DataType, error) {
+	debug.Assert(args[0].ID() == args[1].ID(), "should only be used on the same types")
+	leftType, rightType := args[0].(*arrow.TimestampType), args[1].(*arrow.TimestampType)
+	debug.Assert(leftType.Unit == rightType.Unit, "should match units")
+
+	if (leftType.TimeZone == "" || rightType.TimeZone == "") && (leftType.TimeZone != rightType.TimeZone) {
+		return nil, fmt.Errorf("%w: subtraction of zoned and non-zoned times is ambiguous (%s, %s)",
+			arrow.ErrInvalid, leftType.TimeZone, rightType.TimeZone)
+	}
+
+	return &arrow.DurationType{Unit: rightType.Unit}, nil
+}
+
+var OutputResolveTemporal = exec.NewComputedOutputType(resolveTemporalOutput)
 
 type validityBuilder struct {
 	mem    memory.Allocator
@@ -522,15 +854,6 @@ type execBufBuilder struct {
 	sz     int
 }
 
-func (bldr *execBufBuilder) resize(newcap int) {
-	if bldr.buffer == nil {
-		bldr.buffer = memory.NewResizableBuffer(bldr.mem)
-	}
-
-	bldr.buffer.ResizeNoShrink(newcap)
-	bldr.data = bldr.buffer.Bytes()
-}
-
 func (bldr *execBufBuilder) reserve(additional int) {
 	if bldr.buffer == nil {
 		bldr.buffer = memory.NewResizableBuffer(bldr.mem)
@@ -549,35 +872,6 @@ func (bldr *execBufBuilder) unsafeAppend(data []byte) {
 	bldr.sz += len(data)
 }
 
-func (bldr *execBufBuilder) unsafeAppendN(n int, val byte) {
-	bldr.data[bldr.sz] = val
-	for i := 1; i < n; i *= 2 {
-		copy(bldr.data[bldr.sz+i:], bldr.data[bldr.sz:bldr.sz+i])
-	}
-	bldr.sz += n
-}
-
-func (bldr *execBufBuilder) append(data []byte) {
-	if bldr.sz+len(data) > cap(bldr.data) {
-		bldr.resize(bldr.sz + len(data))
-	}
-	bldr.unsafeAppend(data)
-}
-
-func (bldr *execBufBuilder) appendN(n int, val byte) {
-	bldr.reserve(n)
-	bldr.unsafeAppendN(n, val)
-}
-
-func (bldr *execBufBuilder) advance(n int) {
-	bldr.reserve(n)
-	bldr.sz += n
-}
-
-func (bldr *execBufBuilder) unsafeAdvance(n int) {
-	bldr.sz += n
-}
-
 func (bldr *execBufBuilder) finish() (buf *memory.Buffer) {
 	if bldr.buffer == nil {
 		buf = memory.NewBufferBytes(nil)
@@ -589,12 +883,12 @@ func (bldr *execBufBuilder) finish() (buf *memory.Buffer) {
 	return
 }
 
-type bufferBuilder[T exec.FixedWidthTypes] struct {
+type bufferBuilder[T arrow.FixedWidthType] struct {
 	execBufBuilder
 	zero T
 }
 
-func newBufferBuilder[T exec.FixedWidthTypes](mem memory.Allocator) *bufferBuilder[T] {
+func newBufferBuilder[T arrow.FixedWidthType](mem memory.Allocator) *bufferBuilder[T] {
 	return &bufferBuilder[T]{
 		execBufBuilder: execBufBuilder{
 			mem: mem,
@@ -606,51 +900,21 @@ func (b *bufferBuilder[T]) reserve(additional int) {
 	b.execBufBuilder.reserve(additional * int(unsafe.Sizeof(b.zero)))
 }
 
-func (b *bufferBuilder[T]) resize(newcap int) {
-	b.execBufBuilder.resize(newcap * int(unsafe.Sizeof(b.zero)))
-}
-
 func (b *bufferBuilder[T]) unsafeAppend(value T) {
-	b.execBufBuilder.unsafeAppend(exec.GetBytes([]T{value}))
-}
-
-func (b *bufferBuilder[T]) unsafeAppendN(n int, value T) {
-	data := exec.GetData[T](b.data)[b.len():]
-	b.execBufBuilder.unsafeAdvance(n * int(unsafe.Sizeof(value)))
-	data[0] = value
-	for i := 1; i < n; i *= 2 {
-		copy(data[i:], data[:i])
-	}
+	b.execBufBuilder.unsafeAppend(arrow.GetBytes([]T{value}))
 }
 
 func (b *bufferBuilder[T]) unsafeAppendSlice(values []T) {
-	b.execBufBuilder.unsafeAppend(exec.GetBytes(values))
-}
-
-func (b *bufferBuilder[T]) advance(n int) {
-	b.execBufBuilder.advance(n * int(unsafe.Sizeof(b.zero)))
-}
-
-func (b *bufferBuilder[T]) append(value T) {
-	b.execBufBuilder.append(exec.GetBytes([]T{value}))
+	b.execBufBuilder.unsafeAppend(arrow.GetBytes(values))
 }
 
 func (b *bufferBuilder[T]) len() int { return b.sz / int(unsafe.Sizeof(b.zero)) }
 
-func (b *bufferBuilder[T]) appendN(n int, value T) {
-	b.reserve(n + b.len())
-	b.unsafeAppendN(n, value)
-}
-
-func (b *bufferBuilder[T]) appendSlice(values []T) {
-	b.execBufBuilder.append(exec.GetBytes(values))
-}
-
 func (b *bufferBuilder[T]) cap() int {
 	return cap(b.data) / int(unsafe.Sizeof(b.zero))
 }
 
-func checkIndexBoundsImpl[T exec.IntTypes | exec.UintTypes](values *exec.ArraySpan, upperLimit uint64) error {
+func checkIndexBoundsImpl[T arrow.IntType | arrow.UintType](values *exec.ArraySpan, upperLimit uint64) error {
 	// for unsigned integers, if the values array is larger
 	// than the maximum index value, then there's no need to bounds check
 	isSigned := !arrow.IsUnsignedInteger(values.Type.ID())
@@ -714,3 +978,12 @@ func checkIndexBoundsChunked(values *arrow.Chunked, upperLimit uint64) error {
 	}
 	return nil
 }
+
+func packBits(vals [32]uint32, out []byte) {
+	const batchSize = 32
+	for i := 0; i < batchSize; i += 8 {
+		out[0] = byte(vals[i] | vals[i+1]<<1 | vals[i+2]<<2 | vals[i+3]<<3 |
+			vals[i+4]<<4 | vals[i+5]<<5 | vals[i+6]<<6 | vals[i+7]<<7)
+		out = out[1:]
+	}
+}
