@@ -19,10 +19,10 @@ package ipc
 import (
 	"io"
 
-	"github.com/apache/arrow/go/v10/arrow"
-	"github.com/apache/arrow/go/v10/arrow/arrio"
-	"github.com/apache/arrow/go/v10/arrow/internal/flatbuf"
-	"github.com/apache/arrow/go/v10/arrow/memory"
+	"github.com/apache/arrow/go/v17/arrow"
+	"github.com/apache/arrow/go/v17/arrow/arrio"
+	"github.com/apache/arrow/go/v17/arrow/internal/flatbuf"
+	"github.com/apache/arrow/go/v17/arrow/memory"
 )
 
 const (
@@ -70,6 +70,8 @@ type config struct {
 	compressNP         int
 	ensureNativeEndian bool
 	noAutoSchema       bool
+	emitDictDeltas     bool
+	minSpaceSavings    *float64
 }
 
 func newConfig(opts ...Option) *config {
@@ -77,6 +79,7 @@ func newConfig(opts ...Option) *config {
 		alloc:              memory.NewGoAllocator(),
 		codec:              -1, // uncompressed
 		ensureNativeEndian: true,
+		compressNP:         1,
 	}
 
 	for _, opt := range opts {
@@ -130,9 +133,12 @@ func WithZstd() Option {
 // WithCompressConcurrency specifies a number of goroutines to spin up for
 // concurrent compression of the body buffers when writing compress IPC records.
 // If n <= 1 then compression will be done serially without goroutine
-// parallelization. Default is 0.
+// parallelization. Default is 1.
 func WithCompressConcurrency(n int) Option {
 	return func(cfg *config) {
+		if n <= 0 {
+			n = 1
+		}
 		cfg.compressNP = n
 	}
 }
@@ -160,6 +166,33 @@ func WithDelayReadSchema(v bool) Option {
 	}
 }
 
+// WithDictionaryDeltas specifies whether or not to emit dictionary deltas.
+func WithDictionaryDeltas(v bool) Option {
+	return func(cfg *config) {
+		cfg.emitDictDeltas = v
+	}
+}
+
+// WithMinSpaceSavings specifies a percentage of space savings for
+// compression to be applied to buffers.
+//
+// Space savings is calculated as (1.0 - compressedSize / uncompressedSize).
+//
+// For example, if minSpaceSavings = 0.1, a 100-byte body buffer won't
+// undergo compression if its expected compressed size exceeds 90 bytes.
+// If this option is unset, compression will be used indiscriminately. If
+// no codec was supplied, this option is ignored.
+//
+// Values outside of the range [0,1] are handled as errors.
+//
+// Note that enabling this option may result in unreadable data for Arrow
+// Go and C++ versions prior to 12.0.0.
+func WithMinSpaceSavings(savings float64) Option {
+	return func(cfg *config) {
+		cfg.minSpaceSavings = &savings
+	}
+}
+
 var (
 	_ arrio.Reader = (*Reader)(nil)
 	_ arrio.Writer = (*Writer)(nil)
