@@ -14,19 +14,18 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.arrow.vector;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-
 import org.apache.arrow.memory.ArrowBuf;
 import org.apache.arrow.memory.BufferAllocator;
 import org.apache.arrow.vector.complex.MapVector;
@@ -43,21 +42,21 @@ import org.apache.arrow.vector.types.pojo.ArrowType;
 import org.apache.arrow.vector.types.pojo.Field;
 import org.apache.arrow.vector.types.pojo.FieldType;
 import org.apache.arrow.vector.util.TransferPair;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 public class TestUnionVector {
   private static final String EMPTY_SCHEMA_PATH = "";
 
   private BufferAllocator allocator;
 
-  @Before
+  @BeforeEach
   public void init() {
     allocator = new DirtyRootAllocator(Long.MAX_VALUE, (byte) 100);
   }
 
-  @After
+  @AfterEach
   public void terminate() throws Exception {
     allocator.close();
   }
@@ -69,7 +68,9 @@ public class TestUnionVector {
     uInt4Holder.value = 100;
     uInt4Holder.isSet = 1;
 
-    try (UnionVector unionVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector unionVector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
       unionVector.allocateNew();
 
       // write some data
@@ -96,7 +97,9 @@ public class TestUnionVector {
 
   @Test
   public void testUnionVectorMapValue() throws Exception {
-    try (UnionVector unionVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector unionVector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
       unionVector.allocateNew();
 
       UnionWriter writer = (UnionWriter) unionVector.getWriter();
@@ -157,7 +160,9 @@ public class TestUnionVector {
 
   @Test
   public void testTransfer() throws Exception {
-    try (UnionVector srcVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector srcVector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
       srcVector.allocateNew();
 
       // write some data
@@ -171,7 +176,9 @@ public class TestUnionVector {
       srcVector.setSafe(5, newBitHolder(false));
       srcVector.setValueCount(6);
 
-      try (UnionVector destVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+      try (UnionVector destVector =
+          new UnionVector(
+              EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
         TransferPair pair = srcVector.makeTransferPair(destVector);
 
         // Creating the transfer should transfer the type of the field at least.
@@ -206,7 +213,9 @@ public class TestUnionVector {
 
   @Test
   public void testSplitAndTransfer() throws Exception {
-    try (UnionVector sourceVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector sourceVector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
 
       sourceVector.allocateNew();
 
@@ -256,17 +265,13 @@ public class TestUnionVector {
       assertEquals(false, sourceVector.isNull(9));
       assertEquals(50, sourceVector.getObject(9));
 
-      try (UnionVector toVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+      try (UnionVector toVector =
+          new UnionVector(
+              EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
 
         final TransferPair transferPair = sourceVector.makeTransferPair(toVector);
 
-        final int[][] transferLengths = {{0, 3},
-            {3, 1},
-            {4, 2},
-            {6, 1},
-            {7, 1},
-            {8, 2}
-        };
+        final int[][] transferLengths = {{0, 3}, {3, 1}, {4, 2}, {6, 1}, {7, 1}, {8, 2}};
 
         for (final int[] transferLength : transferLengths) {
           final int start = transferLength[0];
@@ -276,8 +281,10 @@ public class TestUnionVector {
 
           /* check the toVector output after doing the splitAndTransfer */
           for (int i = 0; i < length; i++) {
-            assertEquals("Different data at indexes: " + (start + i) + "and " + i, sourceVector.getObject(start + i),
-                toVector.getObject(i));
+            assertEquals(
+                sourceVector.getObject(start + i),
+                toVector.getObject(i),
+                "Different data at indexes: " + (start + i) + "and " + i);
           }
         }
       }
@@ -286,7 +293,9 @@ public class TestUnionVector {
 
   @Test
   public void testSplitAndTransferWithMixedVectors() throws Exception {
-    try (UnionVector sourceVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector sourceVector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
 
       sourceVector.allocateNew();
 
@@ -345,16 +354,13 @@ public class TestUnionVector {
       assertEquals(false, sourceVector.isNull(9));
       assertEquals(30.5f, sourceVector.getObject(9));
 
-      try (UnionVector toVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+      try (UnionVector toVector =
+          new UnionVector(
+              EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
 
         final TransferPair transferPair = sourceVector.makeTransferPair(toVector);
 
-        final int[][] transferLengths = {{0, 2},
-            {2, 1},
-            {3, 2},
-            {5, 3},
-            {8, 2}
-        };
+        final int[][] transferLengths = {{0, 2}, {2, 1}, {3, 2}, {5, 3}, {8, 2}};
 
         for (final int[] transferLength : transferLengths) {
           final int start = transferLength[0];
@@ -364,7 +370,10 @@ public class TestUnionVector {
 
           /* check the toVector output after doing the splitAndTransfer */
           for (int i = 0; i < length; i++) {
-            assertEquals("Different values at index: " + i, sourceVector.getObject(start + i), toVector.getObject(i));
+            assertEquals(
+                sourceVector.getObject(start + i),
+                toVector.getObject(i),
+                "Different values at index: " + i);
           }
         }
       }
@@ -384,8 +393,9 @@ public class TestUnionVector {
     children.add(new Field("int", FieldType.nullable(MinorType.INT.getType()), null));
     children.add(new Field("varchar", FieldType.nullable(MinorType.VARCHAR.getType()), null));
 
-    final FieldType fieldType = new FieldType(false, new ArrowType.Union(UnionMode.Sparse, typeIds),
-        /*dictionary=*/null, metadata);
+    final FieldType fieldType =
+        new FieldType(
+            false, new ArrowType.Union(UnionMode.Sparse, typeIds), /*dictionary=*/ null, metadata);
     final Field field = new Field("union", fieldType, children);
 
     MinorType minorType = MinorType.UNION;
@@ -395,22 +405,24 @@ public class TestUnionVector {
     assertTrue(vector.getField().equals(field));
 
     // Union has 2 child vectors
-    assertEquals(vector.size(), 2);
+    assertEquals(2, vector.size());
 
     // Check child field 0
     VectorWithOrdinal intChild = vector.getChildVectorWithOrdinal("int");
-    assertEquals(intChild.ordinal, 0);
+    assertEquals(0, intChild.ordinal);
     assertEquals(intChild.vector.getField(), children.get(0));
 
     // Check child field 1
     VectorWithOrdinal varcharChild = vector.getChildVectorWithOrdinal("varchar");
-    assertEquals(varcharChild.ordinal, 1);
+    assertEquals(1, varcharChild.ordinal);
     assertEquals(varcharChild.vector.getField(), children.get(1));
   }
 
   @Test
   public void testGetBufferAddress() throws Exception {
-    try (UnionVector vector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector vector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
       boolean error = false;
 
       vector.allocateNew();
@@ -443,9 +455,8 @@ public class TestUnionVector {
 
       List<ArrowBuf> buffers = vector.getFieldBuffers();
 
-
       try {
-        long offsetAddress = vector.getOffsetBufferAddress();
+        vector.getOffsetBufferAddress();
       } catch (UnsupportedOperationException ue) {
         error = true;
       } finally {
@@ -454,7 +465,7 @@ public class TestUnionVector {
       }
 
       try {
-        long dataAddress = vector.getDataBufferAddress();
+        vector.getDataBufferAddress();
       } catch (UnsupportedOperationException ue) {
         error = true;
       } finally {
@@ -467,7 +478,9 @@ public class TestUnionVector {
 
   @Test
   public void testSetGetNull() {
-    try (UnionVector srcVector = new UnionVector(EMPTY_SCHEMA_PATH, allocator, null)) {
+    try (UnionVector srcVector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
       srcVector.allocateNew();
 
       final NullableIntHolder holder = new NullableIntHolder();
@@ -487,6 +500,36 @@ public class TestUnionVector {
     }
   }
 
+  @Test
+  public void testCreateNewVectorWithoutTypeExceptionThrown() {
+    try (UnionVector vector =
+        new UnionVector(
+            EMPTY_SCHEMA_PATH, allocator, /* field type */ null, /* call-back */ null)) {
+      IllegalArgumentException e1 =
+          assertThrows(IllegalArgumentException.class, () -> vector.getTimeStampMilliTZVector());
+      assertEquals(
+          "No TimeStampMilliTZ present. Provide ArrowType argument to create a new vector",
+          e1.getMessage());
+
+      IllegalArgumentException e2 =
+          assertThrows(IllegalArgumentException.class, () -> vector.getDurationVector());
+      assertEquals(
+          "No Duration present. Provide ArrowType argument to create a new vector",
+          e2.getMessage());
+
+      IllegalArgumentException e3 =
+          assertThrows(IllegalArgumentException.class, () -> vector.getFixedSizeBinaryVector());
+      assertEquals(
+          "No FixedSizeBinary present. Provide ArrowType argument to create a new vector",
+          e3.getMessage());
+
+      IllegalArgumentException e4 =
+          assertThrows(IllegalArgumentException.class, () -> vector.getDecimalVector());
+      assertEquals(
+          "No Decimal present. Provide ArrowType argument to create a new vector", e4.getMessage());
+    }
+  }
+
   private static NullableIntHolder newIntHolder(int value) {
     final NullableIntHolder holder = new NullableIntHolder();
     holder.isSet = 1;
