@@ -14,23 +14,31 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.arrow.vector.table;
 
 import static org.apache.arrow.vector.complex.BaseRepeatedValueVector.OFFSET_WIDTH;
 
+import java.math.BigDecimal;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.List;
-
 import org.apache.arrow.memory.BufferAllocator;
 import org.apache.arrow.vector.BigIntVector;
+import org.apache.arrow.vector.BitVector;
 import org.apache.arrow.vector.BitVectorHelper;
+import org.apache.arrow.vector.DateDayVector;
+import org.apache.arrow.vector.DateMilliVector;
+import org.apache.arrow.vector.DecimalVector;
+import org.apache.arrow.vector.DurationVector;
 import org.apache.arrow.vector.FieldVector;
 import org.apache.arrow.vector.FixedSizeBinaryVector;
 import org.apache.arrow.vector.Float4Vector;
 import org.apache.arrow.vector.Float8Vector;
 import org.apache.arrow.vector.GenerateSampleData;
 import org.apache.arrow.vector.IntVector;
+import org.apache.arrow.vector.IntervalDayVector;
+import org.apache.arrow.vector.IntervalMonthDayNanoVector;
+import org.apache.arrow.vector.IntervalYearVector;
 import org.apache.arrow.vector.LargeVarBinaryVector;
 import org.apache.arrow.vector.LargeVarCharVector;
 import org.apache.arrow.vector.SmallIntVector;
@@ -63,7 +71,9 @@ import org.apache.arrow.vector.complex.impl.UnionMapWriter;
 import org.apache.arrow.vector.complex.writer.Float8Writer;
 import org.apache.arrow.vector.complex.writer.IntWriter;
 import org.apache.arrow.vector.holders.NullableUInt4Holder;
+import org.apache.arrow.vector.types.TimeUnit;
 import org.apache.arrow.vector.types.Types;
+import org.apache.arrow.vector.types.pojo.ArrowType;
 import org.apache.arrow.vector.types.pojo.Field;
 import org.apache.arrow.vector.types.pojo.FieldType;
 
@@ -106,8 +116,8 @@ public class TestUtils {
     IntVector v1 = getSimpleIntVector(allocator);
     VarCharVector v2 = new VarCharVector(VARCHAR_VECTOR_NAME_1, allocator);
     v2.allocateNew(2);
-    v2.set(0, "one".getBytes());
-    v2.set(1, "two".getBytes());
+    v2.set(0, "one".getBytes(StandardCharsets.UTF_8));
+    v2.set(1, "two".getBytes(StandardCharsets.UTF_8));
     v2.setValueCount(2);
     vectorList.add(v1);
     vectorList.add(v2);
@@ -123,8 +133,8 @@ public class TestUtils {
     IntVector v1 = getSimpleIntVector(allocator);
     LargeVarCharVector v2 = new LargeVarCharVector(VARCHAR_VECTOR_NAME_1, allocator);
     v2.allocateNew(2);
-    v2.set(0, "one".getBytes());
-    v2.set(1, "two".getBytes());
+    v2.set(0, "one".getBytes(StandardCharsets.UTF_8));
+    v2.set(1, "two".getBytes(StandardCharsets.UTF_8));
     v2.setValueCount(2);
     vectorList.add(v1);
     vectorList.add(v2);
@@ -133,16 +143,16 @@ public class TestUtils {
 
   /**
    * Returns a list of two FieldVectors to be used to instantiate Tables for testing. The first
-   * vector is an IntVector and the second is a VarBinaryVector. Each vector has two values set.
-   * The large binary vectors values are "one" and "two" encoded with UTF-8
+   * vector is an IntVector and the second is a VarBinaryVector. Each vector has two values set. The
+   * large binary vectors values are "one" and "two" encoded with UTF-8
    */
   static List<FieldVector> intPlusVarBinaryColumns(BufferAllocator allocator) {
     List<FieldVector> vectorList = new ArrayList<>();
     IntVector v1 = getSimpleIntVector(allocator);
     VarBinaryVector v2 = new VarBinaryVector(VARBINARY_VECTOR_NAME_1, allocator);
     v2.allocateNew(2);
-    v2.set(0, "one".getBytes());
-    v2.set(1, "two".getBytes());
+    v2.set(0, "one".getBytes(StandardCharsets.UTF_8));
+    v2.set(1, "two".getBytes(StandardCharsets.UTF_8));
     v2.setValueCount(2);
     vectorList.add(v1);
     vectorList.add(v2);
@@ -151,16 +161,16 @@ public class TestUtils {
 
   /**
    * Returns a list of two FieldVectors to be used to instantiate Tables for testing. The first
-   * vector is an IntVector and the second is a VarBinaryVector. Each vector has two values set.
-   * The large binary vectors values are "one" and "two" encoded with UTF-8
+   * vector is an IntVector and the second is a VarBinaryVector. Each vector has two values set. The
+   * large binary vectors values are "one" and "two" encoded with UTF-8
    */
   static List<FieldVector> intPlusLargeVarBinaryColumns(BufferAllocator allocator) {
     List<FieldVector> vectorList = new ArrayList<>();
     IntVector v1 = getSimpleIntVector(allocator);
     LargeVarBinaryVector v2 = new LargeVarBinaryVector(VARBINARY_VECTOR_NAME_1, allocator);
     v2.allocateNew(2);
-    v2.set(0, "one".getBytes());
-    v2.set(1, "two".getBytes());
+    v2.set(0, "one".getBytes(StandardCharsets.UTF_8));
+    v2.set(1, "two".getBytes(StandardCharsets.UTF_8));
     v2.setValueCount(2);
     vectorList.add(v1);
     vectorList.add(v2);
@@ -169,16 +179,16 @@ public class TestUtils {
 
   /**
    * Returns a list of two FieldVectors to be used to instantiate Tables for testing. The first
-   * vector is an IntVector and the second is a FixedSizeBinary vector. Each vector has two values set.
-   * The large binary vectors values are "one" and "two" encoded with UTF-8
+   * vector is an IntVector and the second is a FixedSizeBinary vector. Each vector has two values
+   * set. The large binary vectors values are "one" and "two" encoded with UTF-8
    */
   static List<FieldVector> intPlusFixedBinaryColumns(BufferAllocator allocator) {
     List<FieldVector> vectorList = new ArrayList<>();
     IntVector v1 = getSimpleIntVector(allocator);
     FixedSizeBinaryVector v2 = new FixedSizeBinaryVector(FIXEDBINARY_VECTOR_NAME_1, allocator, 3);
     v2.allocateNew(2);
-    v2.set(0, "one".getBytes());
-    v2.set(1, "two".getBytes());
+    v2.set(0, "one".getBytes(StandardCharsets.UTF_8));
+    v2.set(1, "two".getBytes(StandardCharsets.UTF_8));
     v2.setValueCount(2);
     vectorList.add(v1);
     vectorList.add(v2);
@@ -196,9 +206,10 @@ public class TestUtils {
 
   /**
    * Returns a list of fixed-width vectors for testing. It includes
+   *
    * <ol>
-   *   <li>all integral and floating point types</li>
-   *   <li>all basic times and timestamps (second, milli, micro, nano</li>
+   *   <li>all integral and floating point types
+   *   <li>all basic times and timestamps (second, milli, micro, nano
    * </ol>
    *
    * The vector names are based on their type name (e.g. BigIntVector is called "bigInt_vector"
@@ -211,8 +222,8 @@ public class TestUtils {
   }
 
   /**
-   * Returns a list of all integral and floating point vectors.
-   * The vector names are based on their type name (e.g. BigIntVector is called "bigInt_vector"
+   * Returns a list of all integral and floating point vectors. The vector names are based on their
+   * type name (e.g. BigIntVector is called "bigInt_vector"
    */
   static List<FieldVector> numericVectors(
       List<FieldVector> vectors, BufferAllocator allocator, int rowCount) {
@@ -247,6 +258,9 @@ public class TestUtils {
     vectors.add(new TimeStampMicroVector("timeStampMicro_vector", allocator));
     vectors.add(new TimeStampNanoVector("timeStampNano_vector", allocator));
 
+    vectors.add(new DateMilliVector("dateMilli_vector", allocator));
+    vectors.add(new DateDayVector("dateDay_vector", allocator));
+
     vectors.forEach(vec -> GenerateSampleData.generateTestData(vec, rowCount));
     return vectors;
   }
@@ -266,6 +280,20 @@ public class TestUtils {
     return vectors;
   }
 
+  static List<FieldVector> intervalVectors(BufferAllocator allocator, int rowCount) {
+    List<FieldVector> vectors = new ArrayList<>();
+    vectors.add(new IntervalDayVector("intervalDay_vector", allocator));
+    vectors.add(new IntervalYearVector("intervalYear_vector", allocator));
+    vectors.add(new IntervalMonthDayNanoVector("intervalMonthDayNano_vector", allocator));
+    vectors.add(
+        new DurationVector(
+            "duration_vector",
+            new FieldType(true, new ArrowType.Duration(TimeUnit.SECOND), null),
+            allocator));
+    vectors.forEach(vec -> GenerateSampleData.generateTestData(vec, rowCount));
+    return vectors;
+  }
+
   /** Returns a list vector of ints. */
   static ListVector simpleListVector(BufferAllocator allocator) {
     ListVector listVector = ListVector.empty(INT_LIST_VECTOR_NAME, allocator);
@@ -291,7 +319,6 @@ public class TestUtils {
     }
     listVector.setLastSet(outerCount - 1);
     listVector.setValueCount(outerCount);
-
     return listVector;
   }
 
@@ -323,7 +350,7 @@ public class TestUtils {
     return structVector;
   }
 
-  /** Returns a MapVector of ints to doubles. */
+  /** Returns a MapVector of longs to doubles. */
   static MapVector simpleMapVector(BufferAllocator allocator) {
     MapVector mapVector = MapVector.empty(BIGINT_INT_MAP_VECTOR_NAME, allocator, false);
     mapVector.allocateNew();
@@ -343,6 +370,24 @@ public class TestUtils {
     return mapVector;
   }
 
+  static List<FieldVector> decimalVector(BufferAllocator allocator, int rowCount) {
+    List<FieldVector> vectors = new ArrayList<>();
+    vectors.add(
+        new DecimalVector(
+            "decimal_vector",
+            new FieldType(true, new ArrowType.Decimal(38, 10, 128), null),
+            allocator));
+    vectors.forEach(vec -> generateDecimalData((DecimalVector) vec, rowCount));
+    return vectors;
+  }
+
+  static List<FieldVector> bitVector(BufferAllocator allocator, int rowCount) {
+    List<FieldVector> vectors = new ArrayList<>();
+    vectors.add(new BitVector("bit_vector", allocator));
+    vectors.forEach(vec -> GenerateSampleData.generateTestData(vec, rowCount));
+    return vectors;
+  }
+
   /** Returns a UnionVector. */
   static UnionVector simpleUnionVector(BufferAllocator allocator) {
     final NullableUInt4Holder uInt4Holder = new NullableUInt4Holder();
@@ -380,4 +425,17 @@ public class TestUtils {
     unionVector.setValueCount(4);
     return unionVector;
   }
+
+  private static void generateDecimalData(DecimalVector vector, int valueCount) {
+    final BigDecimal even = new BigDecimal("0.0543278923");
+    final BigDecimal odd = new BigDecimal("2.0543278923");
+    for (int i = 0; i < valueCount; i++) {
+      if (i % 2 == 0) {
+        vector.setSafe(i, even);
+      } else {
+        vector.setSafe(i, odd);
+      }
+    }
+    vector.setValueCount(valueCount);
+  }
 }
